/* callbacks.c
 *
 * Copyright (C) 2000 Vince (Vincent Saugey), Mandriva <vince@mandriva.com>
 * Copyright (C) 2000,2001,2002 Frdric Crozat, Mandriva <fcrozat@mandriva.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <string.h>
#include <dirent.h>
#include <unistd.h>
#include "callbacks.h"
#include "objmenu.h"
#include "dndctree.h"
#include "eggiconlist.h"

static GSList * menu_list= NULL;
static GSList * app_list= NULL;
static gchar * current_needs;
static int dir_entry=0;  /* the number of directory entry already creating */
static GtkTreeIter extended_attribute_select;
GData ** root = NULL;
static GtkTargetEntry cdnd_target_table =  { "gtk-clist-drag-reorder", 0, 0 } ;
static gboolean updatingUI = FALSE;
static gint status=1;
static gboolean saved = FALSE;
static gboolean system_menu = FALSE;
static gboolean modified = FALSE;
static gboolean simplified = FALSE;
static gboolean should_check_simplified = TRUE;
gboolean should_quit = TRUE;

#define STATUS_DO_NOTHING 1
#define STATUS_SAVING 2
#define STATUS_UPDATING 3

/* WARNING : if you add new files like these in menu directory, 
 * be sure to add them in exclusion list of clean_directory function in objmenu.c
 */
#define DISABLE_MDK_CUSTOMIZATION_FILE "disable_mdk_customization"
#define ENABLE_MDK_CUSTOMIZATION_FILE "enable_mdk_customization"
#define ENABLE_SIMPLIFIED_FILE "enable_simplified"

static void create_extend_attribute(GData ** obj);

static void refresh_ctree_menu();
static void refresh_ctree_app();

static void make_regular_list ();

static int cmp_path (GData ** obj1, GData ** obj2);
static int cmp_title (GData ** obj1, GData ** obj2);
static int cmp_section (GData ** obj1, GData ** obj2);
static int cmp_dir (GData ** obj1, GData ** obj2);
static void create_context_menu(GtkWidget * mainwindow);


extern GSList * available_wm;
extern GtkWidget * get_glade_widget(char * id);
extern gchar * get_current_wm ();

void set_current_needs(gchar * new_needs) {
  current_needs = new_needs;
}

int cmp_with_title(GData** obj, char * title)  {
  return strcmp(TITLE(obj), title);
}

void
set_system_menu_flag(gboolean value) {
  system_menu = value;
}

gboolean
is_system_menu_flag() {
  return system_menu;
}

void
set_modified_flag() {
  /* don't touch flag if we are modifying ui */
  if (!updatingUI)
    modified = TRUE;
}

gboolean
is_modified_flag()
{
  return modified;
}

static void
set_simplified(gboolean value)
{
  simplified = value;
}

gboolean
check_simplified()
{
  if (!should_check_simplified) {
    return simplified;
  }
  else {
    gchar * filename;
    gboolean value = FALSE;

    filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),DISABLE_MDK_CUSTOMIZATION_FILE);
    if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
      value = FALSE;
      g_free(filename);
    }
    else { 
      g_free(filename);
      filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_MDK_CUSTOMIZATION_FILE);
      if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	value = FALSE;
	g_free(filename);
      }
      else {
	g_free(filename);
	filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_SIMPLIFIED_FILE);
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	  value = TRUE;
	  g_free(filename);
	}    
	else {
	  g_free(filename);
	  filename = g_strdup_printf(SYSTEM_MENU_OVERLOAD "/" ENABLE_SIMPLIFIED_FILE);
	  if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	    value = TRUE;
	  }
	  g_free(filename);
	}
      }
    }
    set_simplified(value);
    should_check_simplified = FALSE;
    return value;
  }
  
}


static void
reset_simplified()
{
  should_check_simplified = TRUE;
}

void
gtk_flush()
{
  while (gtk_events_pending()) {
    gtk_main_iteration();
  }
}

void
set_global_menu_list(GSList *menulist, GtkWidget * progressbar) {
  GSList* iterator;
  GtkCList * menu_ctree_as_clist;
  gint nbentries = 0, counter=0;

  menu_ctree_as_clist = GTK_CLIST (get_glade_widget( "ctree_menu"));
  gtk_clist_clear(menu_ctree_as_clist);

  if (progressbar) {
    gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progressbar), 
			     _("Free older menu structure"));
    gtk_flush();
  }

  if (menu_list) {
    g_slist_foreach (menu_list, (GFunc)g_datalist_clear, NULL);
    g_slist_free(menu_list);
  }

  menu_list = menulist;
  nbentries = g_slist_length(menu_list);
  iterator = menu_list;
  root = NULL;

  if (progressbar) {
    gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progressbar), _("Loading icons"));
    gtk_flush();
  }

  while (iterator) {
    counter++;
    create_extend_attribute(iterator->data);
    if (progressbar) {
      gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(progressbar), ((gfloat)counter / (gfloat)(2*nbentries)));
      gtk_flush();
    }
     
    if (strcmp(PATH(iterator->data), "/") == 0) { 
      /* found a root node in the menu list */
      root = iterator->data;
    }
    iterator =  g_slist_next (iterator);
  }

  if (progressbar) {
    gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progressbar), 
				    _("Creating standard menu structure"));
    gtk_flush();
  }

  if (menu_list) {
    menu_list = g_slist_sort(menu_list, (GCompareFunc) cmp_section);  
    make_regular_list();
    menu_list = g_slist_sort(menu_list, (GCompareFunc) cmp_path);
  }

  create_context_menu(get_glade_widget("MenuDrake")); 
  if (root == NULL) { /* let's create a root entry */
    root = g_new (GData*, 1);
    g_datalist_init (root);
    g_datalist_set_data_full(root, "section", g_strdup("/"), g_free);
    g_datalist_set_data_full(root, "title", g_strdup(""), g_free);
    g_datalist_set_data_full(root, "needs", g_strdup("x11"),g_free);
    create_extend_attribute(root);
  }

  if (progressbar) {
    gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progressbar), _("Inserting in tree"));
    gtk_flush();
  }

  refresh_ctree_menu();
}

void
set_global_app_list(GSList *menulist, GtkWidget * progressbar, gfloat offset) {
  GSList * iterator, * tmp;
  int nbentries = 0, counter=0;
  if (progressbar) {
    gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progressbar), _("Free older menu structure"));
    gtk_flush();
  }
  if (app_list) {
    g_slist_foreach (app_list, (GFunc)g_datalist_clear, NULL);
  }
  g_slist_free(app_list);
  iterator = app_list = menulist;
  while (iterator) {
    if (IS_DIR(iterator->data) ||
	(g_ascii_strcasecmp(NEEDS(iterator->data),"x11") && (g_ascii_strcasecmp(NEEDS(iterator->data),"text")))) {
      tmp =  g_slist_next (iterator);
      g_datalist_clear(iterator->data);
      app_list = g_slist_remove_link(app_list,iterator);
      iterator = tmp;
    }
    else {
      nbentries++;
      iterator =  g_slist_next (iterator);
    }
  }
  if (progressbar) {
    gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progressbar), _("Sorting applications"));
    gtk_flush();
  }
  app_list = g_slist_sort (app_list, (GCompareFunc) cmp_title);

  iterator = app_list;
  while (iterator) {
    counter++;
    create_extend_attribute(iterator->data);
    REMOVE(iterator->data, "section");
    g_datalist_set_data_full(iterator->data, "section", g_strdup("/"),g_free);
    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(progressbar), ((gfloat)counter) / ((gfloat)2*nbentries) + offset);
    gtk_flush();
    iterator =  g_slist_next (iterator);
  }
}


/****************** For creating extra attribute (not in the menu file) *********************/
/* 
 *  Menudrake uses many attributes for it own fonctionnement  
 *  Specialy you can found 
 *   pixbuf : ref to a gdkpixbuf (in fact pixbuf are preload for optimize tree building)
 *   pixmap : pointer to a gdkpimap of the entry
 *   mask : pointer to the mask of the pixmap of the entry
 *   path : full name of a widget, often : section/title
 *   expand : a boolean for keep the node open when change of context
 */


static 
GdkPixbuf * load_in_dir(const gchar *dir, const gchar *filename) {
  gchar *full_file_name = g_strdup_printf("%s/%s", dir, filename);
  GdkPixbuf *pixbuf = NULL;
  GError *gerror = NULL;
  if (!access(full_file_name, R_OK)) {
    pixbuf = gdk_pixbuf_new_from_file (full_file_name, &gerror);
    if (gerror) {
      g_error_free (gerror);
    }
  }
  g_free(full_file_name);
  return pixbuf;
}

static 
GdkPixbuf * get_image(const gchar * file) 
{
  /* search the best xpm size */
  static gchar *dir_path [] = { "",
				"/usr/share/icons/large",
				"/usr/share/icons/crystalsvg/32x32/apps", 
				"/usr/share/icons/hicolor/32x32/apps", 
				"/usr/share/icons/locolor/32x32/apps", 
				"/usr/share/icons", 
				"/usr/share/icons/mini", 
				"/usr/share/pixmaps", 
				"/usr/share/icons/crystalsvg/32x32/actions", 
				"/usr/share/icons/hicolor/32x32/actions", 
				"/usr/share/icons/locolor/32x32/actions", 
				"/usr/share/icons/crystalsvg/32x32/filesystems", 
				"/usr/share/icons/hicolor/32x32/filesystems", 
				"/usr/share/icons/locolor/32x32/filesystems", 
				"/usr/share/icons/locolor/16x16/apps", 
				NULL};
  char * * pt = dir_path;
  GdkPixbuf *pixbuf;
  while(*pt) {
    if ((pixbuf = load_in_dir(*pt, file))) {
      return pixbuf;
    }
    pt++;
  }
  return NULL;
}

static 
void loadimage(GData ** obj) {
  GdkPixmap *pixmap = NULL;
  GdkBitmap *mask = NULL;
  GdkPixbuf *pixbuf = NULL;
  GdkPixbuf *pixbuf_scaled = NULL;
  gchar * icon = ICON(obj);
  REMOVE(obj, "pixbuf");
  REMOVE(obj, "pixmap");
  REMOVE(obj, "mask");
  if (icon) {
    pixbuf = get_image(icon);
    if (pixbuf) {
      g_datalist_set_data_full(obj, "pixbuf",
			       pixbuf, (GDestroyNotify) g_object_unref);
      pixbuf_scaled = gdk_pixbuf_scale_simple (pixbuf, 24, 24, GDK_INTERP_BILINEAR);
      gdk_pixbuf_render_pixmap_and_mask (pixbuf_scaled, &pixmap, &mask, 127);
      g_datalist_set_data_full(obj, "pixmap", 
			       pixmap, (GDestroyNotify) g_object_unref);
      if (mask) {
	g_datalist_set_data_full(obj, "mask",
				 mask, (GDestroyNotify) g_object_unref);
      }
      g_object_unref (pixbuf_scaled);
    } 
  } 
  return;
}

static 
void create_path(GData ** obj) {
  if (PATH(obj)) {
    REMOVE(obj, "path");
  }
  if (IS_DIR(obj)) {
    if (! strcmp (SECTION(obj) , "/")) {
      g_datalist_set_data_full(obj, "path", 
			       g_strdup_printf ("%s/", TITLE(obj)), 
			       g_free);
    }
    else {
      g_datalist_set_data_full(obj, "path",
			       g_strdup_printf ("%s%s/", SECTION(obj), TITLE(obj)), 
			       g_free);
    }
  }
  else {
    if (! strcmp (SECTION(obj), "/")) {
      g_datalist_set_data_full(obj, "path", g_strdup(TITLE(obj)),g_free);
    }
    else {
      g_datalist_set_data_full(obj, "path", 
			       g_strdup_printf ("%s%s", SECTION(obj), TITLE(obj)), 
			       g_free);
    }
  }
}


static void 
create_extend_attribute(GData ** obj) {

  g_return_if_fail(obj != NULL);

  create_path(obj);

  if (!ICON(obj)) {
    if (IS_DIR(obj)) {
      g_datalist_set_data_full(obj,"icon",g_strdup("applications_section.png"), g_free);
    }
    else {
      g_datalist_set_data_full(obj,"icon",g_strdup("windowmanager_section.png"), g_free);
    }
  }

  loadimage(obj);
}


/******************* For creating the list of the menu object *************************/
/* 
 * Menudrake need to have a coherent liste of object menu for build his tree
 * All objects needs existe, menudrake cannot accept a virtual obj
 * For exemple a obj which you cannot found the object of the path
 * Here you can found all fonctions for creating the missing object 
 *
 */
static 
int cmp_path (GData ** obj1, 
	      GData ** obj2) {
  return strcmp(PATH(obj1), PATH(obj2));
}

static 
int cmp_dir (GData ** obj1, 
	     GData ** obj2) {
  if (!IS_DIR(obj1) || !IS_DIR(obj2)) {
    return 1;
  }
  return (strcmp (SECTION(obj1), SECTION(obj2)) || strcmp (TITLE(obj1), TITLE(obj2)));
}

static 
int cmp_title (GData ** obj1, 
	       GData ** obj2) {
  return strcmp(TITLE(obj1), TITLE(obj2));
}

static 
int count_depth (gchar * string) {
  int nivel = 2;
  while (*string == '/') {
    string++;
  }
  while (*string) {
    if (*(string++) == '/') {
      nivel++;
    }
  }
  return nivel;
}

static
gboolean needs_in(gchar * father, gchar * children) {
  return !g_ascii_strcasecmp(father, "x11") 
    || !g_ascii_strcasecmp(father, "text") 
    || !g_ascii_strcasecmp(father, "all") 
    || !g_ascii_strcasecmp(father, "wm")
    || !g_ascii_strcasecmp(father,"vc")
    || !g_ascii_strcasecmp(father, children);
}

static
int cmp_section (GData ** obj1, 
		 GData ** obj2) {
  return (strcmp(SECTION(obj1), SECTION(obj2)) 
	  || strcmp(TITLE(obj1), TITLE(obj2)) 
	  || needs_in(NEEDS(obj1), NEEDS(obj2)));
}

static 
gboolean cmp_obj_path(GData** obj1, GData** obj2) {
  return strcmp (SECTION(obj1),SECTION(obj2))
    || strcmp(TITLE(obj1), TITLE(obj2))
    || !needs_in(NEEDS(obj1),NEEDS(obj2));
}

static
gint
cmp_order (GtkCTree *tree, 
	   GtkCTreeRow *node1,
	   GtkCTreeRow *node2)
{
  GData ** parent;
  gchar * order;
  if (node1->parent) {
    parent = gtk_ctree_node_get_row_data(tree, node1->parent);
  }
  else {
    parent = root;
  }
  order = GET(parent, "order");
  if (order) {
    gchar * * tab_order = g_strsplit(order, ",", 0);
    gchar * * pt = tab_order;
    while (*pt) {
      if (!strcmp(TITLE(node1->row.data), *pt)) { g_strfreev(tab_order); return -1;}
      if (!strcmp(TITLE(node2->row.data), *pt)) { g_strfreev(tab_order); return 1;}
      pt ++;
      }
    g_strfreev(tab_order);
    return 0;
  }
  return strcmp (TITLE(node1->row.data), TITLE(node2->row.data));
}


/* Function to complete the list of widget of the list */
#define MAX_BUF 150

static
GSList * create_default_path(gchar * end_section,
			     int depth,
			     gchar * needs) {
  GSList* list = NULL;
  int i;
  gchar buf [MAX_BUF];
  gchar * pt;
  GData ** obj;
  strncpy (buf, end_section, MAX_BUF);
  if (strlen(buf) != 0) {
   pt = buf + strlen (buf)-1;  /* -1 to don't count the last / in a section */
   while ((pt > buf) && (*pt!='/')) {
     pt --;
   }
  }
  else {
    pt = buf;
  }
  if (pt > buf) {
    *pt = '\0';
  }
  i=count_depth(buf) - depth;
  for (pt = buf + strlen (buf);i!=0;i--) {
    while ((pt > buf) && (*pt!='/')) {
      pt --;
    }
    if (pt!=buf) {
      *pt='\0';
    }
    obj = g_new (GData*, 1);
    g_datalist_init (obj);
    if (pt==buf) {
      g_datalist_set_data_full (obj, "section", g_strdup("/"),g_free);
      g_datalist_set_data_full (obj, "title", g_strdup(pt), g_free);
      g_datalist_set_data_full (obj, "needs", g_strdup(needs), g_free);
    }
    else {
      g_datalist_set_data_full (obj, "section",
				g_strdup_printf("%s/",buf), g_free);
      g_datalist_set_data_full (obj, "title", g_strdup(pt+1), g_free);
      g_datalist_set_data_full (obj, "needs", g_strdup(needs), g_free);
    }
    list = g_slist_prepend (list, obj);
  }
  return list;
}

static
void create_elem_path(GData** obj) {
  GSList * sub_list;
  if (!PATH(obj)) {
    return;
  }
  sub_list =  create_default_path(PATH(obj), 1, NEEDS(obj));
  while (sub_list) {
    if (g_slist_find_custom (menu_list, sub_list->data, (GCompareFunc) cmp_obj_path)) {
      g_datalist_clear(sub_list->data);
      sub_list->data = NULL;
    }
    else {
      GSList *old_dir = NULL;
      while ((old_dir = g_slist_find_custom(menu_list, sub_list->data, (GCompareFunc) cmp_dir))) {
	g_datalist_clear(old_dir->data);
	menu_list = g_slist_remove_link(menu_list, old_dir);
      }
      create_extend_attribute(sub_list->data);
      menu_list = g_slist_prepend(menu_list, sub_list->data);
      sub_list->data = NULL;
    }
    sub_list = g_slist_remove_link(sub_list, sub_list);
  }
}

/* function call for init the creation of the object */
static
void make_regular_list () {
  const gchar * section="", * new_section;
  const gchar * needs="", * new_needs;
  GSList * element= menu_list;

  /* create path node for each element and for each section*/
  while (element) {
    new_section = SECTION(element->data);
    new_needs = NEEDS(element->data);
    if (strcmp(new_section, section) || strcmp(new_needs, needs)) {
      create_elem_path(element->data);
    }
    section = new_section;
    needs = new_needs;
    element = g_slist_next(element);
  }
  /* remove duplicated entries in list*/
  element = menu_list = g_slist_sort(menu_list, (GCompareFunc) cmp_path);
  while (element && g_slist_next(element)) {
    while (g_slist_next(element)
	   && (!strcmp(PATH (element->data), PATH (g_slist_next(element)->data))
	   && IS_DIR(element->data) && IS_DIR(g_slist_next(element)->data))) {
      if (needs_in (NEEDS(element->data), NEEDS(g_slist_next(element)->data))) {
	    /* g_slist_next(element) "needs" is included in element "needs", removing duplicated node g_slist_next(element) */
	g_datalist_clear(g_slist_next(element)->data);
	menu_list = g_slist_remove_link(menu_list, g_slist_next(element));
      }
      else {
	      /* element "needs" is included in g_slist_next(element) "needs", removing duplicated node element */
	GSList *new_elem = g_slist_next(element);
	g_datalist_clear(element->data);
	menu_list = g_slist_remove_link(menu_list, element);
	element = new_elem;
      } 
    } 
    element = g_slist_next(element);
  }
}

static
void show_modal_warning_dialog (const char * text) {
  GtkWidget * dialog;
  dialog = gtk_message_dialog_new (GTK_WINDOW (get_glade_widget("MenuDrake")),
				   GTK_DIALOG_DESTROY_WITH_PARENT,
				   GTK_MESSAGE_WARNING,
				   GTK_BUTTONS_OK,
				   text);

  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy (dialog);
}

/********************************* For manipulation of the tree *******************************/

static
GtkCTreeNode* insert_node (GtkCTree* tree, 
			   GtkCTreeNode* parent,
			   GtkCTreeNode* sibling, 
			   GData** obj) {
  GtkCTreeNode* node; 
  gchar * title [2];
  if (TITLE(obj)) {
    title[0] = dgettext(MENU_DOMAIN, TITLE(obj));
  }
  else {
    title[0] = "";
  }
  title[1] = NULL;
  node = gtk_ctree_insert_node  (tree, parent, sibling,
				 title,  /* colon text */ 
				 5,      /* spacing beetween ligne */
				 GET(obj, "pixmap"),   /* pixmap closed */
				 GET(obj, "mask"),     /* mask closed  */
				 GET(obj, "pixmap"),   /* pixmap opened */
				 GET(obj, "mask"),     /* mask opened */
				 !IS_DIR(obj), /* is leaf */
				 (gboolean)GET(obj, "expand"));   /* Is expanded */
  gtk_ctree_node_set_row_data (tree, node, obj);
  g_datalist_set_data (obj, "node", node);
  if (g_ascii_strcasecmp(current_needs,"all") && !g_ascii_strcasecmp(current_needs,NEEDS(obj))) {
    GdkColor color;
    if (gdk_color_parse("blue", &color)) {
      gtk_ctree_node_set_foreground (tree, node, &color);
    }
    else {
      show_modal_warning_dialog (_("couldn't find blue color in /usr/lib/X11/rgb.txt file"));
    }
  }
  return node;
}

/* recursive function which builds the tree */
static 
void add_node (const gchar * path, GSList * * plist, GtkCTreeNode* parent, GtkCTree * tree) {
  GData** obj;
  GtkCTreeNode* node;
  while (*plist) {
    obj = (*plist)->data;
    if (((SECTION(obj) != NULL) && g_str_has_prefix (SECTION(obj), ".hidden")) || strcmp (TITLE(obj), ".hidden") == 0) {
	    *plist =  g_slist_next (*plist);
	    continue;
    }
    if (needs_in(NEEDS(obj),current_needs)) {     
      if (strcmp(SECTION(obj), path)) {
	/* only add leaf to current branch */
	*plist =  g_slist_next (*plist);
	continue;
      }
      node = insert_node(tree, parent, NULL, obj);
      *plist =  g_slist_next (*plist);
      if (IS_DIR(obj)) {
	GSList * list = *plist;
	add_node(PATH(obj),&list , node, tree);
      }
    }
    else { /* don't add node which is not in selected context */
      REMOVE(obj,"node");
      *plist =  g_slist_next (*plist);
    }
  }
}

static 
GtkCTreeNode* get_selected_node(GtkCTree* tree) {
  GList * select = GTK_CLIST(tree)->selection;
  if (select) {
    return GTK_CTREE_NODE(select->data);
  }
  return NULL;
}

static 
GData** get_selected_object(GtkCTree* tree) {
  GtkCTreeNode* node = get_selected_node(tree);
  if (node) {
    return gtk_ctree_node_get_row_data(tree,node);
  }
  else {
    return NULL;
  }
}


/************************************** Functions for the creation of the menu of existing wm **********************************/
/* 
 * The context menu is create with the different value found in the needs attribute of the menu object
 * 
 */
void
remove_context_menu_item(GtkMenuItem * menuitem, 
			 GtkWindow * mainwindow) 
{
  gchar * needs = g_object_get_data (G_OBJECT(menuitem),"user_data");
  g_object_set_data(G_OBJECT (mainwindow), needs, NULL);
}

void 
on_activate_context_menu     (GtkMenuItem     *menuitem,
			      gpointer         user_data);

void
create_context_menu(GtkWidget * mainwindow) 
{
  GtkMenuItem * context_menu = GTK_MENU_ITEM(get_glade_widget("Context"));
  GtkWidget * item = NULL;
  GtkWidget * all_item = NULL;
  GtkMenu * menu = NULL;
  GSList * radio_group = NULL;
  gboolean default_set = FALSE;
  GSList * wm;

  if (!context_menu) {
    g_error ("Main window %p does not contain context attribute", mainwindow); 
  }
  if (g_object_get_data(G_OBJECT(mainwindow),"all")) {
    GtkMenu * menu = GTK_MENU(context_menu->submenu);
    g_list_foreach(GTK_MENU_SHELL(menu)->children, (GFunc) remove_context_menu_item, mainwindow);
    gtk_menu_item_remove_submenu (context_menu);
  }
  menu = GTK_MENU (gtk_menu_new());
  item = gtk_radio_menu_item_new_with_label(radio_group, _("All environments"));
  all_item = item;
  g_object_set_data (G_OBJECT(item),"user_data","all");
  radio_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
  if (!g_ascii_strcasecmp("all",current_needs)) {
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), TRUE);
      default_set = TRUE;
  }
  gtk_signal_connect (GTK_OBJECT(item), "activate", G_CALLBACK(on_activate_context_menu), 
		      mainwindow);
  gtk_menu_append (GTK_MENU (menu), item);
  g_object_set_data(G_OBJECT (mainwindow), "all", item);
  for (wm = available_wm ; wm ; wm = g_slist_next(wm)) {
    char * needs;
    if (strlen(wm->data) == 0) {
      continue;
    }
    needs = g_strdup(wm->data);
    item = gtk_radio_menu_item_new_with_label(radio_group, needs);
    g_object_set_data_full (G_OBJECT (item),"user_data", needs , g_free);
    radio_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
    if (!g_ascii_strcasecmp(needs,current_needs)) {
      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), TRUE);
      default_set = TRUE;
    }
    gtk_signal_connect (GTK_OBJECT(item), "activate", G_CALLBACK (on_activate_context_menu), 
			mainwindow);
    gtk_menu_append (GTK_MENU (menu), item);
    g_object_set_data(G_OBJECT (mainwindow), needs, item);
  }
  gtk_widget_show_all(GTK_WIDGET(menu));
  gtk_menu_item_set_submenu (context_menu, GTK_WIDGET(menu));
  if (!default_set) { /* wm autodetection was not useful*/
    set_current_needs("all");
    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (all_item), TRUE);
  }
}


/**************************  Add, delete, move obj in tree and menu list **********************/

static
void del_obj(GData ** obj, GtkCTree* tree) {
  GtkCTreeNode* node = GET(obj, "node");
  if (tree && node) {
     gtk_ctree_remove_node (tree, node);
  }
  g_datalist_clear (obj);
}

static 
void del_node (const gchar * path, GSList * * plist, GtkCTree * tree) {
  GData** obj;
  while (plist && *plist) {
    obj = (*plist)->data;
    if (strcmp(SECTION(obj), path)) {
      return;
    }
    *plist =  g_slist_next (*plist);
    if (IS_DIR(obj)) {
      del_node(PATH(obj),plist, tree);
    }
    menu_list = g_slist_remove(menu_list, obj);
    del_obj(obj,tree);
  }
}


static
GtkCTreeNode * insert_obj(GtkCTree *tree, GData**obj) {
  GtkCTreeNode * parent=NULL,* sibling=NULL;
  if (!GTK_CLIST(tree)->selection) {
    g_datalist_set_data_full(obj, "section", g_strdup("/"),g_free);
    parent = NULL;
    sibling = NULL;
  }
  else {
    GData ** select_obj;
    parent = GTK_CLIST(tree)->selection->data;
    select_obj = gtk_ctree_node_get_row_data(tree,parent);
    if (IS_DIR(select_obj)) {
      g_datalist_set_data_full(obj, "section", g_strdup(PATH(select_obj)),g_free);
      sibling = NULL;
    }
    else {
      g_datalist_set_data_full(obj, "section", g_strdup(SECTION(select_obj)),g_free);
      sibling = parent;
      parent = GTK_CTREE_ROW(parent)->parent;
    }
  }
  create_extend_attribute(obj);
  menu_list = g_slist_insert_sorted (menu_list, obj, (GCompareFunc) cmp_path);
  sibling = insert_node(tree,parent,sibling,obj);
  gtk_clist_sort(GTK_CLIST(tree));
  if (parent) {
    gtk_ctree_expand(tree, parent);
  }
  gtk_ctree_select(tree, sibling);
  refresh_ctree_app();
  set_modified_flag();
  return sibling;
}

static
void new_obj(GtkCTree * tree,
	     gchar * title, 
	     gchar * command) {
  GData ** obj = g_new (GData*, 1);
  g_datalist_init (obj);
  g_datalist_set_data_full(obj, "title", g_strdup(title), g_free);
  if (g_ascii_strcasecmp("all",current_needs)) {
    g_datalist_set_data_full(obj, "needs",g_strdup(current_needs),g_free);
  }
  else {
    g_datalist_set_data_full(obj, "needs",g_strdup("x11"),g_free);
  }
  if (command) {
    g_datalist_set_data_full (obj, "command", g_strdup(command), g_free);
  }
  insert_obj(tree, obj);
}

static
GSList* move_obj(GData**obj, GData **parent) {
  gchar * new_section, *old_path;
  if (parent) {
    new_section = PATH(parent);
  }
  else {
    new_section = "/";
  }
  REMOVE(obj, "section");
  g_datalist_set_data_full(obj,"section",g_strdup(new_section), g_free);
  if (IS_DIR(obj)) {
    GSList *jump, *iterator = g_slist_find(menu_list, obj);
    old_path = g_strdup(PATH(obj));
    create_path(obj);
    iterator = g_slist_next (iterator);
    while (iterator && !strcmp(old_path,SECTION(iterator->data))){
      jump = move_obj(iterator->data, obj);
      if (jump) {
	iterator = jump;
      }
      else {
	iterator = g_slist_next (iterator);
      }
    }
    g_free(old_path);
    return iterator;
  }
  else {
    create_path(obj);
  }
  return NULL;
} /* don't forget to sort the list before go out this fonction */

static 
void build_order (GtkCTree *ctree, GtkCTreeNode *node, char* order) {
  strcat (order, ",");
  strcat (order, TITLE(gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(node))));
}


/***************** function for refresh the ctree view ************************/

static void
refresh_ctree_menu() {
  GSList *list;
  GtkCTree* menu_tree = GTK_CTREE(get_glade_widget("ctree_menu"));
  gchar * title;
 
  for (list = menu_list; list; list = g_slist_next(list)) {
    REMOVE(list->data,"node");
  }
 
  gtk_clist_freeze(GTK_CLIST(menu_tree));
  gtk_clist_clear(GTK_CLIST(menu_tree));

  list = menu_list;
  add_node("/", &list, NULL, menu_tree);  

  refresh_ctree_app();
 
  if (g_ascii_strcasecmp(current_needs,"all") == 0) {
    title = g_strdup_printf(_("Environment : %s"), _("All environments"));
  }
  else {
    title = g_strdup_printf(_("Environment : %s"), current_needs);
  }
  gtk_clist_set_column_title (GTK_CLIST(menu_tree), 0, title);
  g_free(title);
  gtk_clist_sort (GTK_CLIST(menu_tree));
  gtk_clist_thaw (GTK_CLIST(menu_tree));
}

static void
refresh_entry(GtkCTree *ctree, GtkCTreeNode *node) {
  GData** obj = gtk_ctree_node_get_row_data(ctree, node);  
  gtk_ctree_set_node_info (ctree, node, 
			   (TITLE(obj)[0] != '\0') ? dgettext(MENU_DOMAIN, TITLE(obj)) : "",  /* title */ 
			   5,      /* spacing beetween ligne */
			   GET(obj, "pixmap"),   /* pixmap closed */
			   GET(obj, "mask"),     /* mask closed  */
			   GET(obj, "pixmap"),   /* pixmap opened */
			   GET(obj, "mask"),     /* mask opened */
			   !IS_DIR(obj), /* is leaf */
			   (gboolean)GET(obj, "expand"));   /* Is expanded */
}

static 
void refresh_ctree_app() {

  GtkWidget *ctree_apps, *ctree_menus;
  GSList *list;
  int row;
  ctree_apps = get_glade_widget( "ctree_apps");
  ctree_menus = get_glade_widget( "ctree_menu");
  row = gtk_clist_find_row_from_data(GTK_CLIST(ctree_apps), 
				     get_selected_object(GTK_CTREE(ctree_apps)));
  gtk_clist_freeze(GTK_CLIST(ctree_apps));
  list = app_list;
  while (list) {
    gint col = gtk_clist_find_row_from_data(GTK_CLIST(ctree_apps), list->data);
    gboolean in_menu = (gboolean)gtk_ctree_find_by_row_data_custom(GTK_CTREE(ctree_menus), 
								   NULL, 
								   TITLE(list->data), 
								   (GCompareFunc)cmp_with_title);
    if (!in_menu) {
      if (col == -1) {
	insert_node(GTK_CTREE(ctree_apps), NULL, NULL, list->data);
      }
    }
    else {
      if (col != -1) {
	gtk_clist_remove (GTK_CLIST(ctree_apps), col);
      }
    }
    list = g_slist_next(list);
  }
  gtk_clist_sort (GTK_CLIST(ctree_apps));

  /* unselect current selection */
  if (gtk_clist_get_selectable(GTK_CLIST(ctree_apps), row)) {
    gtk_clist_select_row(GTK_CLIST(ctree_apps), row, 0);
  }
  gtk_clist_thaw (GTK_CLIST(ctree_apps));

}


/********** other way not very clean need to be change ************/
static 
void refresh_order(GtkCTree *ctree,
		   GtkCTreeNode *node) {
  char order [100000];
  char * begin;
  GData ** obj;
  order[0]='\0';
  if (node) {
    obj = gtk_ctree_node_get_row_data(ctree,node);
  }
  else {
    obj = root;
  }
  gtk_ctree_pre_recursive_to_depth(ctree, node,
				   node==NULL?1:GTK_CTREE_ROW(node)->level + 1,
				   (GtkCTreeFunc)build_order, order);
  begin = ((node != NULL)? index(order+1,',') : order);
  REMOVE(obj, "order");
  g_datalist_set_data_full(obj, "order", g_strdup(begin+1), g_free);
}


/************************* order function **************************/
static
gboolean gtk_search_for_path (GtkCTree* ctree,
			      GtkCTreeNode* source_node,
			      GtkCTreeNode* new_parent,
			      GtkCTreeNode* new_sibling) {
  gboolean found = FALSE;
  int level;
  void search_path (GtkCTree *ctree, GtkCTreeNode *node, GData** obj1) {
    if (!(found)) {
      GData** obj2 = gtk_ctree_node_get_row_data(ctree, node);
      if (level != GTK_CTREE_ROW(node)->level) {
	if (node!=NULL && !needs_in(NEEDS(obj2), NEEDS(obj1))) {
	  found = TRUE;
	}
      }
      else {
	if ((obj1 != obj2) && !strcmp(TITLE(obj1),TITLE(obj2))) {
	  found = TRUE;
	}
      }
    }
  }
  
  level = (new_parent==NULL)?1:GTK_CTREE_ROW(new_parent)->level + 1;
  gtk_ctree_pre_recursive_to_depth(ctree, new_parent, level, (GtkCTreeFunc)search_path, 
				   gtk_ctree_node_get_row_data(ctree, source_node));
  return !found;
}


/************************************* The callbacks *********************************************/

void
on_system_menu_clicked                 (GtkWidget       *widget,
                                        gpointer         user_data)
{
  set_system_menu_flag(TRUE);	
  if (widget != NULL) {
    gtk_widget_hide_all(get_glade_widget("rootmenu_dialog"));
  }
  updatingUI = TRUE;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (get_glade_widget("systemmenu_radiobutton")),TRUE);
  updatingUI = FALSE;
   if (widget != NULL) /* called by callback */ {
     should_quit = FALSE;
     gtk_main_quit();
   }
}

void
on_root_menu_clicked                 (GtkWidget       *widget,
				      gpointer         user_data)
{
  set_system_menu_flag(FALSE);
  if (widget != NULL) {
    gtk_widget_hide_all(get_glade_widget("rootmenu_dialog"));
  }
  updatingUI = TRUE;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (get_glade_widget("rootmenu_radiobutton")),TRUE);
  updatingUI = FALSE;
  if (widget != NULL) /* called by callback */ {
    should_quit = FALSE;
    gtk_main_quit();
  }
}

/* 
 * return TRUE if yes (ie continue) has been selected 
 */
static
gboolean show_not_saved_dialog_if_needed() {
  if (is_modified_flag()) {
    GtkWidget * dialog;
    gint result;
    
    dialog = gtk_message_dialog_new (GTK_WINDOW (get_glade_widget("MenuDrake")),
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_MESSAGE_WARNING,
				     GTK_BUTTONS_NONE,
				     _("Your changes have not been saved. Do you want to continue?"));
    gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_YES, GTK_RESPONSE_YES);
    gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_NO, GTK_RESPONSE_NO);
    result = gtk_dialog_run (GTK_DIALOG (dialog));
    saved = (result == GTK_RESPONSE_YES);
    gtk_widget_destroy (dialog);
    return saved;
  }
  else {
    return TRUE;
  }
}

void
on_systemmenu_radiobutton_toggled                 (GtkWidget       *widget,
						   gpointer         user_data)
{
  gtk_widget_hide(get_glade_widget("follow_system_settings_radio_button"));
  if ((updatingUI) || !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(get_glade_widget("systemmenu_radiobutton")))) {
    return;
  }

  if (show_not_saved_dialog_if_needed()) {
    set_system_menu_flag(TRUE);
    gtk_menu_item_activate(GTK_MENU_ITEM(get_glade_widget("reload")));
  }
  else {
    updatingUI = TRUE;

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (get_glade_widget("rootmenu_radiobutton")),TRUE);
    updatingUI = FALSE;
  }
}

void
on_rootmenu_radiobutton_toggled                 (GtkWidget       *widget,
						 gpointer         user_data)
{
  gtk_widget_show(get_glade_widget("follow_system_settings_radio_button"));

  if ((updatingUI) || !GTK_TOGGLE_BUTTON(get_glade_widget("rootmenu_radiobutton"))->active) {
    return;
  }

  if (show_not_saved_dialog_if_needed()) {
    set_system_menu_flag(FALSE);
    gtk_menu_item_activate(GTK_MENU_ITEM(get_glade_widget("reload")));
  }
  else {
    updatingUI = TRUE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (get_glade_widget("systemmenu_radiobutton")),TRUE);
    updatingUI = FALSE;
  }
}

void 
reload_menu_style () {
  GtkToggleButton * mdk_customization_togglebutton;
  gchar * filename;

  /* let's change mdk_menu_customization menu entry */
  if (is_system_menu_flag()) { /* we are root */
    mdk_customization_togglebutton = GTK_TOGGLE_BUTTON (get_glade_widget("mandrake_radio_button"));

    filename = g_strdup(SYSTEM_MENU_OVERLOAD "/" ENABLE_SIMPLIFIED_FILE);
    if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
      set_simplified(TRUE);
      mdk_customization_togglebutton = GTK_TOGGLE_BUTTON (get_glade_widget("simplified_radio_button"));
    }
    else {
      g_free(filename);
      filename = g_strdup(SYSTEM_MENU_OVERLOAD "/" DISABLE_MDK_CUSTOMIZATION_FILE);
      if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	set_simplified(FALSE);
	mdk_customization_togglebutton = GTK_TOGGLE_BUTTON (get_glade_widget("original_radio_button"));
      }
    }
    g_free(filename);
  }
  else {
    mdk_customization_togglebutton = GTK_TOGGLE_BUTTON (get_glade_widget("follow_system_settings_radio_button"));

    filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_SIMPLIFIED_FILE);
    if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
      mdk_customization_togglebutton = GTK_TOGGLE_BUTTON (get_glade_widget("simplified_radio_button"));
    }
    g_free(filename);

    filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),DISABLE_MDK_CUSTOMIZATION_FILE);
    if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
      mdk_customization_togglebutton = GTK_TOGGLE_BUTTON (get_glade_widget("original_radio_button"));
    }
    g_free(filename);

    filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_MDK_CUSTOMIZATION_FILE);
    if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
      mdk_customization_togglebutton = GTK_TOGGLE_BUTTON (get_glade_widget("mandrake_radio_button"));
    }
    g_free(filename);

  }

  updatingUI = TRUE;
  gtk_toggle_button_set_active(mdk_customization_togglebutton, TRUE);   
  updatingUI = FALSE;
  
}

void
on_menu_style_cancel                   (GtkWidget       *widget,
					gpointer         user_data)
{
  reload_menu_style();
}

void
on_menudrake_realize                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GtkWidget *ctree;
 

  ctree = get_glade_widget( "ctree_menu");
  gtk_clist_set_row_height (GTK_CLIST(ctree), 26);
  GTK_CLIST_SET_FLAG (GTK_CLIST(ctree), CLIST_REORDERABLE);

  gtk_clist_set_compare_func (GTK_CLIST(ctree), (GtkCListCompareFunc) cmp_order);
  gtk_drag_dest_set (GTK_WIDGET(ctree),
		     GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
		     &cdnd_target_table, 1, GDK_ACTION_MOVE);
  gtk_ctree_set_drag_compare_func (GTK_CTREE(ctree), gtk_search_for_path);

  ctree = get_glade_widget( "ctree_apps");
  gtk_clist_set_row_height (GTK_CLIST(ctree), 26);
  gtk_drag_source_set (GTK_WIDGET(ctree), GDK_BUTTON1_MASK,
		       &cdnd_target_table, 1,
		       GDK_ACTION_MOVE);

  reload_menu_style();
}


/*********************************** callbacks for ctree ****************************************/

static 
void refresh_extended_attribute(GtkListStore *list, GData**obj) {
  GSList * commun_list, * iterator, * no_commun_list = NULL;
  static char * list_commun_attrib [] = { "title", "longtitle", "command", "section", "icon", "needs", NULL };
  char ** pt;
  gboolean commun;
  GtkTreeIter iter;
  
  /* create list of no common attribute */
  commun_list = get_attributes_list();
  while (commun_list) {
    commun = FALSE;
    for (pt = list_commun_attrib; *pt != NULL; pt++) {
      if (!strcmp(*pt,(gchar*)commun_list->data)) {
	commun=TRUE;
      }
    }
    if (!commun) {
      no_commun_list = g_slist_prepend(no_commun_list, commun_list->data);
    }
    commun_list = g_slist_next(commun_list);
  }
  /* build list of extended argument */
  gtk_list_store_clear(list);
  iterator = no_commun_list;
  while (iterator) {
    if (GET(obj, iterator->data)) {      
      gtk_list_store_append (list, &iter);
      gtk_list_store_set (list, &iter, 
		      EXTENDED_ATTRIBUTES_COLUMN, (gchar*) iterator->data, 
		      EXTENDED_VALUES_COLUMN, (gchar*) GET(obj, iterator->data), 
		      -1);
    }
    iterator = g_slist_next(iterator);
  }
  g_slist_free(no_commun_list);
}


void
on_ctree_menu_tree_select_row          (GtkCTree        *ctree,
                                        GList           *node,
                                        gint             column,
                                        gpointer         user_data)
{
  GData**obj;
  GtkEntry  *entry;
  GtkWidget *button;
  gchar *new_obj;
  GtkWidget *button_pixmap;

  gtk_widget_set_sensitive(get_glade_widget( "notebook_property"), TRUE);
  obj = get_selected_object(ctree);
  entry = GTK_ENTRY(get_glade_widget( "entry_title"));
  new_obj = TITLE(obj);
  /* modifying UI => don't touch modified flag */
  updatingUI = TRUE;
  if (new_obj && (new_obj[0] != '\0')) {
    gtk_entry_set_text(entry,g_strdup (dgettext (MENU_DOMAIN, new_obj)));
  }
  else
    gtk_entry_set_text(entry, "");

  entry = GTK_ENTRY(get_glade_widget( "entry_long_title"));
  new_obj =  GET(obj, "longtitle");
  if (new_obj && (new_obj[0] != '\0')) {
    gtk_entry_set_text(entry, g_strdup (dgettext (MENU_DOMAIN, new_obj)));
  }
  else {
    gtk_entry_set_text(entry, "");
  }
								      
  if (IS_DIR(obj)) {
    gtk_widget_hide(get_glade_widget( "entry_hbox"));
    gtk_widget_hide(get_glade_widget( "label_command"));
    gtk_widget_hide(get_glade_widget( "checkbutton_needs"));
  }
  else {
    entry = GTK_ENTRY(get_glade_widget( "entry_command"));
    gtk_entry_set_text(entry,g_strdup (GET(obj,"command")));
    gtk_widget_show(get_glade_widget ("entry_hbox"));
    gtk_widget_show(get_glade_widget( "label_command"));
    gtk_widget_show(GTK_WIDGET(button=get_glade_widget( "checkbutton_needs")));

    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), !g_ascii_strcasecmp(NEEDS(obj),"text"));    
  }
  /* And now extended argument */
  refresh_extended_attribute(GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW(get_glade_widget( "list_extended_arg")))), obj);
  button_pixmap = get_glade_widget( "button_pixmap");
  if (GET(obj,"pixbuf")) {
    GtkWidget * icon;
    GdkPixbuf * scaled = gdk_pixbuf_scale_simple (GET(obj,"pixbuf"), 48, 48, GDK_INTERP_BILINEAR);
    icon = gtk_image_new_from_pixbuf(scaled);
    g_object_unref (scaled);
    gtk_misc_set_padding (GTK_MISC(icon),4,4);
    gtk_widget_destroy(GTK_BIN(button_pixmap)->child);
    gtk_container_add (GTK_CONTAINER(button_pixmap), icon);
  }
  else {
    gtk_widget_destroy(GTK_BIN(button_pixmap)->child);
    gtk_container_add (GTK_CONTAINER(button_pixmap), gtk_label_new(_("No icon")));
  }
  gtk_widget_show_all(button_pixmap);
  updatingUI = FALSE;
}


void
on_ctree_menu_tree_unselect_row        (GtkCTree        *ctree,
                                        GList           *node,
                                        gint             column,
                                        gpointer         user_data)
{
  gtk_widget_set_sensitive(get_glade_widget( "notebook_property"),FALSE);
}



gboolean
on_ctree_menu_button_press_event       (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  
  if (event->type == GDK_BUTTON_PRESS) {
    GdkEventButton *bev = (GdkEventButton *)event;

    if (bev->button == 3) {
      int row, column;

      if (gtk_clist_get_selection_info (GTK_CLIST (widget), bev->x, bev->y,
					&row, &column)) {
	gtk_clist_select_row(GTK_CLIST(widget), row, column);
      }
      gtk_menu_popup(GTK_MENU(get_glade_widget("menu_popup")), NULL, NULL, NULL, NULL,
                     bev->button, bev->time);
      gtk_widget_set_sensitive(get_glade_widget("delete"), (gboolean)get_selected_node(GTK_CTREE(widget)));
      return TRUE;
    }
  }
  return FALSE;
}


void
on_ctree_menu_tree_expand              (GtkCTree        *ctree,
                                        GList           *node,
                                        gpointer         user_data)
{
  GData ** obj = gtk_ctree_node_get_row_data(ctree,GTK_CTREE_NODE(node));
  refresh_entry(ctree, GTK_CTREE_NODE(node));
  g_datalist_set_data(obj, "expand", (gpointer)TRUE);  
}


void
on_ctree_menu_tree_collapse            (GtkCTree        *ctree,
                                        GList           *node,
                                        gpointer         user_data)
{
  GData ** obj = gtk_ctree_node_get_row_data(ctree,GTK_CTREE_NODE(node));
  REMOVE(obj, "expand");
}

void
on_ctree_menu_row_move                 (GtkCTree        *ctree,
                                        GList           *node,
                                        GList           *new_parent,
                                        GList           *new_sibling,
                                        gpointer         user_data)
{
  move_obj(gtk_ctree_node_get_row_data(ctree,GTK_CTREE_NODE(node)),
	   gtk_ctree_node_get_row_data(ctree,GTK_CTREE_NODE(new_parent)));
  set_modified_flag();
  refresh_order(ctree, GTK_CTREE_NODE(new_parent));
  menu_list = g_slist_sort  (menu_list, (GCompareFunc) cmp_path);
  gtk_ctree_sort_recursive (ctree, GTK_CTREE_NODE(new_parent));
}


/******************************************************************************/
/************************************* Callback for menu **********************/
/******************************************************************************/
void
on_activate_context_menu     (GtkMenuItem     *menuitem,
			      gpointer         user_data)
{
  /* only refresh tree for selected menu item */
  if (GTK_CHECK_MENU_ITEM(menuitem)->active) {
    set_current_needs (g_object_get_data(G_OBJECT(menuitem),"user_data"));
    refresh_ctree_menu();
  }
}

static
gint wait_end_child(GtkWidget * progress)
{
  gint end; 
  int pid;
  
  pid = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (progress), "pid"));

  if (waitpid (pid, &end, WNOHANG)!=0) {
    GtkStatusbar * statusbar = GTK_STATUSBAR(get_glade_widget("statusbar"));
    if (WIFEXITED(end)) {
      gtk_statusbar_push (statusbar, STATUS_DO_NOTHING, 
			  _("Update menu configuration finished"));
    }
    else {
      gtk_statusbar_push (statusbar, STATUS_DO_NOTHING, 
			  _("Update menu configuration finished with error !!!"));
    }
    gtk_widget_set_sensitive(get_glade_widget( "button_save"),TRUE);
    gtk_widget_set_sensitive(get_glade_widget( "save"),TRUE);
    status = STATUS_DO_NOTHING;
    gtk_widget_destroy(g_object_get_data (G_OBJECT (progress),
					  "dialog"));
    if (get_current_wm() != NULL && (strcmp(get_current_wm(),"kde") == 0)) {
      g_spawn_command_line_async ("dcop kded kbuildsycoca recreate",NULL);
    }

    return FALSE;
  }
  gtk_progress_bar_pulse (GTK_PROGRESS_BAR(progress));
  return TRUE;
}


void
on_update_activate () {
  GtkWidget *statusbar;
  statusbar = get_glade_widget( "statusbar");
  if (status == STATUS_DO_NOTHING) {
    GtkWidget *progress, *dialog;
    gchar * cmd;
    char **argv;
    int argc;
    int pid = -1;

    GError *error = NULL;
    status = STATUS_UPDATING;
    statusbar = get_glade_widget( "statusbar");
    gtk_statusbar_push (GTK_STATUSBAR(statusbar), STATUS_UPDATING,
			_("Update menu configuration"));
    gtk_flush();
    cmd = g_strdup("/usr/bin/update-menus -n");
    if (!is_system_menu_flag()) {
      g_strconcat(cmd," -u",NULL);
    }
    g_shell_parse_argv (cmd, &argc, &argv, &error);

    if (error) {
      g_print(error->message);
      g_error_free (error);
      error = NULL;
      return;
    }
    g_free (cmd);
    g_spawn_async_with_pipes (NULL,argv, NULL, 
			      G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_STDERR_TO_DEV_NULL, 
			      NULL, NULL, &pid, NULL, NULL, NULL, &error);
    g_strfreev (argv);
    if (error) {
      g_print(error->message);
      g_error_free (error);
      show_modal_warning_dialog (_("cannot update menu"));
      gtk_statusbar_push (GTK_STATUSBAR(statusbar), STATUS_DO_NOTHING, 
			  _("Error updating menu"));
      return;
    }
    dialog = gtk_dialog_new ();
    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
    progress = gtk_progress_bar_new();
    gtk_signal_connect (GTK_OBJECT(dialog), "delete_event", G_CALLBACK (gtk_true), NULL);
    gtk_window_set_modal(GTK_WINDOW (dialog), TRUE); 
    gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 5);

    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		       gtk_label_new (_("Update menu configuration")));
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
		       progress);
    gtk_progress_bar_pulse(GTK_PROGRESS_BAR (progress));
    gtk_widget_show_all(dialog);
    g_object_set_data (G_OBJECT (progress), "pid", GINT_TO_POINTER(pid));
    g_object_set_data (G_OBJECT (progress), "dialog", dialog);
    gtk_timeout_add (100,(GtkFunction) wait_end_child, progress); 
  }
  else 
    show_modal_warning_dialog (_("Menu generation already in progress"));
}

void
on_reload_menus_activate                     (GtkMenuItem     *menuitem,
					      gpointer         user_data)
{
  GtkWidget * progresswindow;
  GtkWidget * progressbar;
  GSList *menus;
  

  if (!show_not_saved_dialog_if_needed()) {
    return;
  }

  load_translation_map();

  if (menuitem == GTK_MENU_ITEM(get_glade_widget("reload"))) {
    menus = create_menu(is_system_menu_flag());
  }
  else{ 
    if (menuitem == GTK_MENU_ITEM(get_glade_widget("reload_system_menu"))) {
      menus = is_system_menu_flag() ? create_lib_menu() : create_menu(TRUE);
    }
    else {
      show_modal_warning_dialog("callback is not linked to correct menu item");
      return;    
    }
  }

  progresswindow = get_glade_widget("progresswindow");
  gtk_widget_show_all(progresswindow);
  progressbar = get_glade_widget( "progressbar");
  set_global_menu_list(menus, progressbar);
  gtk_widget_set_sensitive(get_glade_widget( "notebook_property"), FALSE);
  gtk_widget_hide_all(progresswindow);
}


void
on_directory_activate                  (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GtkCTree *ctree; 
  char *title = NULL;
  ctree = GTK_CTREE(get_glade_widget( "ctree_menu"));
  title = g_strdup_printf(_("Submenu %d"), ++dir_entry);
  while (gtk_ctree_find_by_row_data_custom(ctree, NULL, title, (GCompareFunc)cmp_with_title)) {
      g_free(title);
      title = g_strdup_printf(_("Submenu %d"), ++dir_entry);
    }
  new_obj(ctree, title, NULL);
  g_free(title);
}


void
on_entry_create                (GtkWidget       *widget,
				gpointer         user_data)
{
  gtk_entry_set_text(GTK_ENTRY(get_glade_widget ("new_title_entry")),"");
  gtk_entry_set_text(GTK_ENTRY(get_glade_widget ("new_command_entry")),"");
  gtk_widget_show_all (get_glade_widget ("create_entry_dialog"));
}


void on_new_entry_validate  (GtkWidget       *widget,
			     gpointer         user_data)
{
  gchar *title, *command;

  if (gtk_entry_get_text(GTK_ENTRY(get_glade_widget ("new_title_entry"))) == NULL 
      || gtk_entry_get_text(GTK_ENTRY(get_glade_widget ("new_command_entry"))) == NULL
      || strlen(gtk_entry_get_text(GTK_ENTRY(get_glade_widget ("new_title_entry")))) == 0
      || strlen(gtk_entry_get_text(GTK_ENTRY(get_glade_widget ("new_command_entry")))) == 0)
  {
    show_modal_warning_dialog (_("Title and Command fields must not be empty"));
    return;
  }

  title = g_strdelimit(g_strdup(gtk_entry_get_text(GTK_ENTRY(get_glade_widget ("new_title_entry")))),"\n",' ');
  command = g_strdelimit(g_strdup(gtk_entry_get_text(GTK_ENTRY(get_glade_widget ("new_command_entry")))),"\n",' ');

  new_obj(GTK_CTREE(get_glade_widget( "ctree_menu")),
	  title, command);
  gtk_widget_hide (get_glade_widget ("create_entry_dialog"));
}

void
on_delete_activate                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GtkCTree* ctree;
  GList * clist;
  GData** select_obj; 
  GSList * list_element;

  ctree = GTK_CTREE(get_glade_widget( "ctree_menu"));
  clist = GTK_CLIST(ctree)->selection;
  if (!clist) {
    show_modal_warning_dialog(_("No node to delete"));
    return;
  }

  select_obj = gtk_ctree_node_get_row_data(ctree,clist->data);
  if (!select_obj) {
    g_error("GtkCTreeNode hasn't associed data");
    return;
  }

  list_element = g_slist_find_custom (menu_list, select_obj, (GCompareFunc) cmp_obj_path);
  if (!list_element) {
    show_modal_warning_dialog(_("No data found for selected node"));
    return;
  }
  list_element = list_element->next;
  del_node(PATH(select_obj) ,&list_element, GTK_CTREE(ctree));
  menu_list = g_slist_remove(menu_list, select_obj);
  del_obj(select_obj,ctree);
  set_modified_flag();
  refresh_ctree_app();
}

static
GData** copy_obj(GData ** obj) {
  GSList * list_attributes = get_attributes_list();
  GData ** copy = g_new (GData*,1);
  g_datalist_init(copy);
  while (list_attributes) {
    if (GET(obj, list_attributes->data)) {
      g_datalist_set_data_full(copy, list_attributes->data, 
			       g_strdup(GET(obj, list_attributes->data)), g_free);
    }
    list_attributes = g_slist_next(list_attributes);
  }
  if (GET(obj, "package")) {
    g_datalist_set_data_full(copy, "package",
			     g_strdup(GET(obj, "package")), g_free);
  }
  
  return copy;
}

void
on_syst_application_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkCTree *menu_ctree;
  GData **obj, **copy;
  menu_ctree = GTK_CTREE(get_glade_widget("ctree_menu"));
  obj= get_selected_object(GTK_CTREE(get_glade_widget("ctree_apps")));
  if (obj) {
    copy = copy_obj(obj);
    if (g_ascii_strcasecmp(current_needs, NEEDS(copy))) {
      REMOVE(copy,"needs");
      if (g_ascii_strcasecmp(current_needs, "all")) {
	g_datalist_set_data_full(copy,"needs", g_strdup(current_needs), g_free);
      }
      else {
	g_datalist_set_data_full(copy,"needs", g_strdup("x11"), g_free);
      }
    }
    insert_obj(menu_ctree, copy);
  }
  else {
    show_modal_warning_dialog(_("Select a system application before"));
  }
}

gboolean
on_ctree_apps_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if (event->type == GDK_BUTTON_PRESS) {
    GdkEventButton *bev = (GdkEventButton *)event;
    if ((bev->button == 3) && (GTK_CLIST(get_glade_widget( "ctree_apps"))->rows)) {
      GtkMenu* Menu = GTK_MENU(gtk_menu_new());
      GtkMenuItem* MenuItem = GTK_MENU_ITEM(gtk_menu_item_new_with_label(_("Add to menu")));

      gtk_menu_append(GTK_MENU(Menu), GTK_WIDGET(MenuItem));
      gtk_signal_connect(GTK_OBJECT(MenuItem), "activate",
                     (GtkSignalFunc) on_syst_application_activate,
                     user_data);
      gtk_widget_show(GTK_WIDGET(MenuItem));
      gtk_widget_show(GTK_WIDGET(Menu));
      gtk_menu_popup(Menu, NULL, NULL, NULL, NULL,
                     bev->button, bev->time);
      return TRUE;
    }
  }
  if (event->type == GDK_2BUTTON_PRESS) {
    on_syst_application_activate(NULL, user_data);
    return TRUE;
  }
  return FALSE;
}

void
on_save_activate                       (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GSList * list;
  GtkWidget * statusbar;
  if (status == STATUS_DO_NOTHING) {
    status = STATUS_SAVING;
    modified = FALSE;
    gtk_widget_set_sensitive(get_glade_widget( "button_save"),FALSE);
    gtk_widget_set_sensitive(get_glade_widget( "save"),FALSE);
    statusbar = get_glade_widget("statusbar");

    list = g_slist_copy (menu_list);
    gtk_statusbar_push (GTK_STATUSBAR(statusbar), STATUS_SAVING, _("Save menu configuration"));
    gtk_flush();
    if (is_system_menu_flag()) {
        save_system_menu(list);
    }
    else {
        save_user_menu(list);
    }
    gtk_statusbar_push (GTK_STATUSBAR(statusbar), STATUS_DO_NOTHING, _("Menu configuration saved"));
    /* warning do not forget the previous function destroy the list */
    status = STATUS_DO_NOTHING;
    on_update_activate ();

  }
  else {
    show_modal_warning_dialog (_("Saving already in progress"));
  }
}

static 
void show_mandrake_customization_warning () {
  GtkWidget * dialog;

  dialog = gtk_message_dialog_new (GTK_WINDOW (get_glade_widget("MenuDrake")),
				   GTK_DIALOG_DESTROY_WITH_PARENT,
				   GTK_MESSAGE_INFO,
				   GTK_BUTTONS_OK,
				   _("Menu Style Changes will be effective at your next login."));

  /* Destroy the dialog when the user responds to it (e.g. clicks a button) */
  g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
			    G_CALLBACK (gtk_widget_destroy),
			    GTK_OBJECT (dialog));
}


void
on_mandrake_customization_show	(GtkWidget	*widget,
				 gpointer	user_data)
{
  gtk_widget_show (get_glade_widget("menu_style_dialog"));
}

void
on_mandrake_customization_activate	(GtkWidget	*widget,
					 gpointer	user_data)
{	
  gchar * filename;
  FILE * file;
  gboolean old_simplified;

  if (updatingUI) {
    return;
  }
  
  old_simplified = check_simplified();

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (get_glade_widget("follow_system_settings_radio_button")))) {
    reset_simplified();
    /* remove disable mdk customization file for user */
    filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),DISABLE_MDK_CUSTOMIZATION_FILE);
    if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
      unlink (filename);
    }
    g_free(filename);

    /* remove enable mdk customization file for user */
    filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_MDK_CUSTOMIZATION_FILE);
    if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
      unlink (filename);
    }
    g_free(filename);

    /* remove enable simplified file for user */
    filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_SIMPLIFIED_FILE);
    if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
      unlink (filename);
    }
    g_free(filename);

    show_mandrake_customization_warning();
  }

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (get_glade_widget("mandrake_radio_button")))) {
    set_simplified(FALSE);
    if (is_system_menu_flag()) /* system wide setting */
      {
	/* remove enable simplified menu file */
	filename = g_strdup_printf(SYSTEM_MENU_OVERLOAD "/" ENABLE_SIMPLIFIED_FILE);
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	  unlink (filename);
	}
	g_free(filename);
	
	filename = g_strdup(SYSTEM_MENU_OVERLOAD "/" DISABLE_MDK_CUSTOMIZATION_FILE);
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	  unlink (filename);
	}
	g_free(filename);
      }
    else /* user wide setting */ {
      filename = g_strdup_printf("%s/.menu",g_get_home_dir());
      if (access(filename, R_OK|W_OK) == -1) {
	if (mkdir (filename, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH) == -1)
	  g_error (_("cannot create directory:%s"), filename);
      }
      g_free(filename);

      /* remove disable mdk customization file for user */
      filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),DISABLE_MDK_CUSTOMIZATION_FILE);
      if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	unlink (filename);
      }
      g_free(filename);

      /* remove disable simplified menu file for user */
      filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_SIMPLIFIED_FILE);
      if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	    unlink (filename);
      }
      g_free(filename);

      /* create enable mdk customization */
      filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_MDK_CUSTOMIZATION_FILE);
      file = fopen(filename,"w");
      fclose(file);
      g_free(filename);
    }
    show_mandrake_customization_warning();
  }

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (get_glade_widget("original_radio_button")))) {
    set_simplified(FALSE);
    /* disabling mandrake customization */
    if (is_system_menu_flag())  /* system wide setting */ {
      /* remove enable simplified menu file */
      filename = g_strdup_printf(SYSTEM_MENU_OVERLOAD "/" ENABLE_SIMPLIFIED_FILE);
      if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	unlink (filename);
      }
      g_free(filename);

      /* create disable mdk customization */
      filename = g_strdup(SYSTEM_MENU_OVERLOAD "/" DISABLE_MDK_CUSTOMIZATION_FILE);
      file = fopen(filename,"w");
      fclose(file);
      g_free(filename);
    }
    else/* user wide setting */
      {
	filename = g_strdup_printf("%s/.menu",g_get_home_dir());
	if (access(filename, R_OK|W_OK) == -1) {
	  if (mkdir (filename, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH) == -1) {
	    g_error (_("cannot create directory:%s"), filename);
	  }
	}
	g_free(filename);

	/* remove enable simplified menu file for user */
	filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_SIMPLIFIED_FILE);
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	  unlink (filename);
	}
	g_free(filename);

	/* remove enable mdk customization file for user */
	filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_MDK_CUSTOMIZATION_FILE);
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	  unlink (filename);
	}
	g_free(filename);

	/* create disable mdk customization */
	filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),DISABLE_MDK_CUSTOMIZATION_FILE);
	file = fopen(filename,"w");
	fclose(file);
	g_free(filename);
      }
    show_mandrake_customization_warning();
  }

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (get_glade_widget("simplified_radio_button")))) {
    set_simplified(TRUE);
    /* enable simplified menu */

    if (is_system_menu_flag())  /* system wide setting */ {
      /* remove enable simplified menu file */
      filename = g_strdup_printf(SYSTEM_MENU_OVERLOAD "/" DISABLE_MDK_CUSTOMIZATION_FILE);
      if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	unlink (filename);
      }
      g_free(filename);

      /* create disable mdk customization */
      filename = g_strdup(SYSTEM_MENU_OVERLOAD "/" ENABLE_SIMPLIFIED_FILE);
      file = fopen(filename,"w");
      fclose(file);
      g_free(filename);
    }
    else/* user wide setting */
      {
	filename = g_strdup_printf("%s/.menu",g_get_home_dir());
	if (access(filename, R_OK|W_OK) == -1) {
	  if (mkdir (filename, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH) == -1) {
	    g_error (_("cannot create directory:%s"), filename);
	  }
	}
	g_free(filename);

	/* remove disable mdk customization for user */
	filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),DISABLE_MDK_CUSTOMIZATION_FILE);
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	  unlink (filename);
	}
	g_free(filename);

	/* remove enable mdk customization file for user */
	filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_MDK_CUSTOMIZATION_FILE);
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
	  unlink (filename);
	}
	g_free(filename);

	/* create simplified menu for user */
	filename = g_strdup_printf("%s/.menu/%s",g_get_home_dir(),ENABLE_SIMPLIFIED_FILE);
	file = fopen(filename,"w");
	fclose(file);
	g_free(filename);
      }
    show_mandrake_customization_warning();
  }

  gtk_widget_hide (get_glade_widget("menu_style_dialog"));
  if (old_simplified != check_simplified()) {
    on_reload_menus_activate (GTK_MENU_ITEM(get_glade_widget("reload")), NULL);
  }
}

void
on_menu_style_clicked	(GtkWidget	*widget,
			 gpointer	user_data) {

  if (updatingUI) {
    return;
  }
  set_modified_flag();
}


/******************** Callbaks for property sheet *********************/

static
void refresh_section (GtkCTree *ctree, GtkCTreeNode *node, char* new_section) {
  GData**obj = gtk_ctree_node_get_row_data(ctree, node);
  REMOVE(obj, "section");
  if (GTK_CTREE_ROW(node)->parent) {
     g_datalist_set_data_full(obj, "section",
			      g_strdup( PATH(gtk_ctree_node_get_row_data(ctree, GTK_CTREE_ROW(node)->parent))),
			      g_free);
  }
  else {
     g_datalist_set_data_full(obj, "section",
			      g_strdup(PATH(root)),
			      g_free);
  }
  create_path (obj);
}

void
on_entry_title_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkCTreeNode * node;
  GData ** obj;
  GtkCTree * tree;


  g_return_if_fail(editable != NULL);
  if (updatingUI)
    return;

  tree = GTK_CTREE(get_glade_widget("ctree_menu"));
  node = get_selected_node(tree);
  if (node) {
    obj = get_selected_object(tree);
    if (obj) {
      const char * new_title = gtk_entry_get_text (GTK_ENTRY(editable));
      GSList * iterator= menu_list;
      if (strcmp(new_title,  dgettext (MENU_DOMAIN, TITLE(obj)))) {
	set_modified_flag();
	while (iterator) {
	  GData ** cur_obj = iterator->data;
	  if (IS_DIR(cur_obj) && !(strcmp (SECTION(obj),SECTION(cur_obj))
				   || strcmp(new_title, TITLE(cur_obj))
				   || !needs_in(NEEDS(obj),NEEDS(cur_obj)))) {
	    show_modal_warning_dialog (_("cannot change path to another entry with the same name"));
	    return;
	  }
	  iterator = g_slist_next(iterator);
	}
	REMOVE(obj, "title");
	new_title = g_strdelimit (g_strdup(gtk_entry_get_text (GTK_ENTRY(editable))),"\n",' ');
	g_datalist_set_data_full(obj, "title", (char *) new_title, g_free);
	refresh_entry(tree,node);
	create_path(obj);	
	gtk_ctree_pre_recursive_to_depth(tree, node,-1,
					  (GtkCTreeFunc)refresh_section, NULL);	
	refresh_order(tree, GTK_CTREE_ROW(node)->parent);
	refresh_ctree_app();
      }
    }    
    return ;   
  }
  show_modal_warning_dialog ("Title changed for an anonymous node");
}


void
on_entry_command_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
  
  GData ** obj;
  gchar *str;

  g_return_if_fail(editable != NULL);
  obj = get_selected_object(GTK_CTREE(get_glade_widget("ctree_menu")));
  if (obj) {
    str = g_strdelimit(g_strdup (gtk_entry_get_text (GTK_ENTRY(editable))),"\n",' ');
    g_datalist_set_data_full(obj, "command", str, g_free);
    set_modified_flag();
  }
  else {
    show_modal_warning_dialog ("Command changed for an anonymous object");
  }
}

void
on_entry_long_title_changed            (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GData ** obj;
  gchar * old_title;
  gchar * new_title;
  g_return_if_fail(editable != NULL);

  obj = get_selected_object(GTK_CTREE(get_glade_widget("ctree_menu")));
  if (!obj) {
    show_modal_warning_dialog ("Long title changed for an anonymous object");
    return;
  }

  old_title = GET(obj, "longtitle");
  if (!old_title || g_ascii_strcasecmp(dgettext (MENU_DOMAIN, old_title) ,gtk_entry_get_text (GTK_ENTRY(editable)))) {
    set_modified_flag();
   new_title = g_strdelimit(g_strdup (gtk_entry_get_text (GTK_ENTRY(editable))),"\n",' ');
    g_datalist_set_data_full(obj, "longtitle", new_title, g_free);
  }
}

void
on_checkbutton_needs_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
  GData ** obj = get_selected_object(GTK_CTREE(get_glade_widget("ctree_menu")));
  if (updatingUI)
    return;

  if (!obj) {
    show_modal_warning_dialog ("Needs changed for an anonymous object");
    return;
  }
  set_modified_flag();
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button))) {
    g_datalist_set_data_full(obj, "needs", g_strdup("text"),g_free);
  }
  else {    
    g_datalist_set_data_full(obj, "needs", g_ascii_strcasecmp(current_needs, "all") ? g_strdup(current_needs) : g_strdup("x11"),g_free);
  }
}


/*******************************************************************/
/****************** function popup callback ************************/
/*******************************************************************/
void
on_ok_attribute_clicked (GtkWidget* widget, GtkWidget *window) 
{
  gchar * name;
  const char* new_attribute; 
  char *clean_attribute, *new_value;
  GtkListStore* store; 
  GtkWidget* property_edit;
  GData **obj;
  
  set_modified_flag();
  property_edit = get_glade_widget ("property_edit");
  name = (gchar*) g_object_get_data (G_OBJECT (property_edit), "name");
  store = g_object_get_data (G_OBJECT (property_edit), "store");
  obj = g_object_get_data (G_OBJECT (property_edit), "obj");
  
  new_attribute = gtk_entry_get_text(GTK_ENTRY(get_glade_widget( "entry_attribute")));
  if ((name && strcmp(name, new_attribute) && GET(obj, new_attribute))
      || (!name && GET(obj, new_attribute))
      || (strlen(new_attribute) == 0)) { 
    /* existing a other attribute for the name */
    gtk_widget_hide(property_edit);
    return;      
  }

  if (name) { 
    REMOVE(obj, name);
  }

  clean_attribute = g_strdelimit(g_strdup (new_attribute),"\n",' ');
  new_value = g_strdelimit(g_strdup (gtk_entry_get_text(GTK_ENTRY(get_glade_widget( "entry_value")))),"\n",' ');
  g_datalist_set_data_full(obj,
			   clean_attribute,
			   new_value,
			   g_free);
  add_attributes_to_list(clean_attribute);
  gtk_widget_hide(property_edit);
  refresh_extended_attribute(GTK_LIST_STORE(store), obj);
}
    
void
show_attribute_window (GData ** obj, gchar* name, gchar* value, GtkTreeView* list)
{
  GtkWidget *attribute;
  GtkEntry *entry_attribute;
  GtkEntry *entry_value;
  entry_attribute = GTK_ENTRY(get_glade_widget("entry_attribute"));
  entry_value = GTK_ENTRY(get_glade_widget("entry_value"));
  attribute = get_glade_widget("property_edit"),

  g_object_set_data (G_OBJECT (attribute), "name", name);
  g_object_set_data (G_OBJECT (attribute), "obj", obj);
  g_object_set_data (G_OBJECT (attribute), "store", gtk_tree_view_get_model(list));

  if (name) {
    gtk_entry_set_text (entry_attribute, g_strdup (name)); 
    gtk_entry_set_text (entry_value, g_strdup (value)); 
  }
  else {
    gtk_entry_set_text (entry_attribute,"");
    gtk_entry_set_text (entry_value,"");
  }
  gtk_widget_show_all(attribute);
}

void
on_add_attribute_activate                  (GtkWidget     *widget,
					    GtkWidget     *window)
{
  GtkCTree *ctree; 
  GtkTreeView *list;
  GData **obj; 
  ctree = GTK_CTREE(get_glade_widget( "ctree_menu"));
  list = GTK_TREE_VIEW(get_glade_widget( "list_extended_arg"));
  obj = get_selected_object(ctree);
  show_attribute_window (obj, NULL, NULL, list);
}

void
on_modify_attribute_activate                  (GtkWidget     *widget,
					       GtkWidget     *window)
{
  GtkWidget *ctree, *list;
  GData **obj;
  gchar *name , *value;
  GtkTreeModel *model;

  ctree = get_glade_widget( "ctree_menu");
  list = get_glade_widget( "list_extended_arg");
  obj = get_selected_object(GTK_CTREE(ctree));

  model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));

  gtk_tree_model_get (model, &extended_attribute_select, 
		  	EXTENDED_ATTRIBUTES_COLUMN, &name,
			EXTENDED_VALUES_COLUMN, &value,
			-1);
  show_attribute_window (obj, name, value, GTK_TREE_VIEW(list));
}

void
on_delete_attribute_activate                  (GtkWidget     *widget,
					       GtkWidget     *window)
{
  GtkWidget *ctree, *list;
  GData **obj;
  gchar *name;
  GtkTreeModel *model;

  ctree = get_glade_widget( "ctree_menu");
  list = get_glade_widget( "list_extended_arg");
  obj = get_selected_object(GTK_CTREE(ctree));

  model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));

  gtk_tree_model_get (model, &extended_attribute_select, 
		  	EXTENDED_ATTRIBUTES_COLUMN, &name,
			-1);

  REMOVE(obj, name);
  gtk_list_store_remove (GTK_LIST_STORE (model), &extended_attribute_select);
  set_modified_flag ();
}

gboolean
on_list_extended_arg_button_press_event(GtkWidget      *widget,
					 GdkEventButton  *event,
					 gpointer        user_data)
{
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));

  if (event->type == GDK_BUTTON_PRESS) {
    GdkEventButton *bev = (GdkEventButton *)event;

    if (bev->button == 3) {
      GtkMenu* Menu;
      GtkMenuItem* MenuItem;
      GtkTreePath *path;
      
      if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),bev->x,bev->y, &path, NULL, NULL, NULL)) {
	gtk_tree_selection_select_path (selection, path);
	gtk_tree_path_free (path);
      }
      else {
	gtk_tree_selection_unselect_all (selection);
      }

      Menu = GTK_MENU(gtk_menu_new());
      MenuItem = GTK_MENU_ITEM(gtk_menu_item_new_with_label(_("Add attribute")));
      gtk_menu_append(GTK_MENU(Menu), GTK_WIDGET(MenuItem));
      gtk_signal_connect(GTK_OBJECT(MenuItem), "activate",
			 (GtkSignalFunc) on_add_attribute_activate,
			 NULL);
      gtk_widget_show(GTK_WIDGET(MenuItem));
      MenuItem = GTK_MENU_ITEM(gtk_menu_item_new_with_label(_("Modify attribute")));
      gtk_menu_append(GTK_MENU(Menu), GTK_WIDGET(MenuItem));
      gtk_signal_connect(GTK_OBJECT(MenuItem), "activate",
		       (GtkSignalFunc) on_modify_attribute_activate,
			 NULL);
      gtk_widget_set_sensitive(GTK_WIDGET(MenuItem), gtk_tree_selection_get_selected (selection, NULL, NULL));
      gtk_widget_show(GTK_WIDGET(MenuItem));
      MenuItem = GTK_MENU_ITEM(gtk_menu_item_new_with_label(_("Delete attribute")));
      gtk_menu_append(GTK_MENU(Menu), GTK_WIDGET(MenuItem));
      gtk_signal_connect(GTK_OBJECT(MenuItem), "activate",
			 (GtkSignalFunc) on_delete_attribute_activate,
			 NULL);
      gtk_widget_set_sensitive(GTK_WIDGET(MenuItem), gtk_tree_selection_get_selected (selection, NULL, NULL));
      gtk_widget_show(GTK_WIDGET(MenuItem));
      gtk_widget_show(GTK_WIDGET(Menu));
      gtk_menu_popup(Menu, NULL, NULL, NULL, NULL,
		     bev->button, bev->time);
      return TRUE;
    }
  }

  if (event->type == GDK_2BUTTON_PRESS) {
    if (gtk_tree_selection_get_selected (selection, NULL, NULL)) {
      /* fix Mdk bug 483 : we got grab and we should release it before displaying the dialog */
      gdk_display_pointer_ungrab(gdk_display_get_default (), GDK_CURRENT_TIME);
      on_modify_attribute_activate(NULL, user_data);
    }
    return TRUE;
  }
  return FALSE;
}

void
on_list_extended_arg_select_row       (GtkTreeSelection *selection, gpointer data)
{
  gtk_tree_selection_get_selected (selection, NULL, &extended_attribute_select);
}

void
on_new_command_browse_clicked (GtkButton *button,
			   gpointer user_data)
{
  GtkWidget *file_selector = NULL;
  gchar * old_file;

  file_selector = gtk_file_chooser_dialog_new  (_("Choose program to run"),
		  GTK_WINDOW(get_glade_widget("menudrake")),
		  GTK_FILE_CHOOSER_ACTION_OPEN,
		  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	          GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
  	          NULL);

 /* Ensure that the dialog box is destroyed when the user clicks a button. */
 
  old_file = g_find_program_in_path(g_filename_from_utf8 (gtk_entry_get_text(GTK_ENTRY (get_glade_widget ("new_command_entry"))),-1,NULL,NULL,NULL));
  if (g_file_test(old_file,G_FILE_TEST_EXISTS)) {
    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (file_selector),
  				   old_file);
  }
  g_free (old_file);

  if (gtk_dialog_run (GTK_DIALOG (file_selector)) == GTK_RESPONSE_ACCEPT) {
    gtk_entry_set_text(GTK_ENTRY (get_glade_widget ("new_command_entry")),
		       g_filename_to_utf8 (gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_selector)),-1,NULL,NULL,NULL));
  }
  gtk_widget_destroy (file_selector);
}

void
on_command_browse_clicked (GtkButton *button,
			   gpointer user_data)
{
  GtkWidget *file_selector = NULL;
  gchar * old_file;

  file_selector = gtk_file_chooser_dialog_new  (_("Choose program to run"),
		  GTK_WINDOW(get_glade_widget("menudrake")),
		  GTK_FILE_CHOOSER_ACTION_OPEN,
		  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	          GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
  	          NULL);

 /* Ensure that the dialog box is destroyed when the user clicks a button. */
 
  old_file = g_find_program_in_path(g_filename_from_utf8 (gtk_entry_get_text(GTK_ENTRY (get_glade_widget ("entry_command"))),-1,NULL,NULL,NULL));
  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (file_selector),
				 old_file);
  g_free (old_file);

  if (gtk_dialog_run (GTK_DIALOG (file_selector)) == GTK_RESPONSE_ACCEPT) {
    gtk_entry_set_text(GTK_ENTRY (get_glade_widget ("entry_command")),
		       g_filename_to_utf8 (gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_selector)),-1,NULL,NULL,NULL));
  }
  gtk_widget_destroy (file_selector);
}

/***************************** Call back for the icon selector ****************************/

GtkWidget *icon_list = NULL;

void
change_icon(GtkWidget *button, GtkWidget *windows) {
  GtkCTree *ctree = GTK_CTREE(get_glade_widget( "ctree_menu"));
  GData **obj = get_selected_object(ctree);
  GList * selected_item;

  set_modified_flag();
  REMOVE(obj, "icon");
  selected_item = egg_icon_list_get_selected_items (EGG_ICON_LIST (icon_list));
  if (selected_item && selected_item->data) {
    char *icon_path;
    GtkTreeIter iter;
    gtk_tree_model_get_iter (egg_icon_list_get_model (EGG_ICON_LIST (icon_list)), &iter, selected_item->data);
    gtk_tree_model_get (egg_icon_list_get_model (EGG_ICON_LIST (icon_list)), &iter, 2, &icon_path, -1);
    g_datalist_set_data_full(obj, "icon", icon_path, g_free);
  }
  g_list_foreach (selected_item, (GFunc)gtk_tree_path_free, NULL);
  g_list_free (selected_item);

  loadimage(obj);
  refresh_entry(ctree,get_selected_node(ctree));
  gtk_ctree_select (ctree, get_selected_node(ctree));
  gtk_widget_hide_all(get_glade_widget( "icon_selector"));
}

static void
fill_icon_list (GObject *object)
{
  GdkPixbuf *pixbuf;
  GDir *directory;
  gchar * file;
  gchar * filename;
  gchar * path;
  GtkTreeIter iter;

  gtk_list_store_clear (GTK_LIST_STORE(egg_icon_list_get_model (EGG_ICON_LIST (icon_list))));


  path = g_object_get_data(object, "user_data");
  if ((path == NULL) || (icon_list == NULL))
    return;

  directory = g_dir_open (path, 0, NULL);
  if (directory == NULL) 
    return;
  
  while ((file = (char *) g_dir_read_name (directory)) != NULL) {
    filename = g_build_filename (path, file, NULL);
    if ((g_str_has_suffix(filename, ".png") ||
	 g_str_has_suffix(filename, ".xpm")) &&
	 g_file_test (filename,  G_FILE_TEST_IS_REGULAR)) {
      pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 48, 48, NULL);
      if (pixbuf) {
 	gtk_list_store_prepend (GTK_LIST_STORE(egg_icon_list_get_model (EGG_ICON_LIST (icon_list))), &iter);
	gtk_list_store_set (GTK_LIST_STORE(egg_icon_list_get_model (EGG_ICON_LIST (icon_list))), &iter,
			0, pixbuf,
			1, file,
			2, filename,
			-1);
      }
    }
    }
}

void
show_directory_picker (GObject *object, gpointer user_data) 
{
 GtkWidget *directory_picker = NULL;

 directory_picker =  gtk_file_chooser_dialog_new(_("Choose a directory containing icons"),
		 	GTK_WINDOW (get_glade_widget("icon_selector")),
				      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
				      NULL);

 if (gtk_dialog_run (GTK_DIALOG (directory_picker)) == GTK_RESPONSE_ACCEPT)
  {
    char *filename;

    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (directory_picker));
    if (filename != NULL) {
	GtkWidget *item;

    	item = gtk_menu_item_new ();
	g_object_set_data(G_OBJECT(item), "user_data", filename);
	g_object_set_data(G_OBJECT(item), "icon_list", icon_list);
        fill_icon_list (G_OBJECT(item));
    }
    g_free (filename);
  }

 gtk_widget_destroy (directory_picker);

}

gint tree_sort_by_file (GtkTreeModel *model,
                         GtkTreeIter *a,
                         GtkTreeIter *b,
                         gpointer user_data)
{
	char *file_a, *file_b;

	gtk_tree_model_get (model, a, 1, &file_a, -1);
        gtk_tree_model_get (model, b, 1, &file_b, -1);
	return strcoll (file_a, file_b);
}


void
on_button_pixmap_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  static char * icons_path [] = {"/usr/share/icons/", 
				 "/usr/share/icons/crystalsvg/32x32/apps",
				 "/usr/share/pixmaps/", 
				 NULL};
  static char * path_name [] =  {"Mandrakelinux", 
				 "KDE",
				 "GNOME", 
				 NULL};
  GtkWidget *selector =  get_glade_widget("icon_selector");
  GtkOptionMenu *option_path;
  GtkWidget * menu, *scrolled_window;
  GtkWidget *item;
  char ** parser_path, ** parser_name;

  if (icon_list == NULL) {
    GtkListStore *store;

    icon_list = egg_icon_list_new ();
    store = gtk_list_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
    gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (store), tree_sort_by_file, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);

    egg_icon_list_set_model (EGG_ICON_LIST (icon_list), GTK_TREE_MODEL (store));
    egg_icon_list_set_selection_mode (EGG_ICON_LIST (icon_list), 
				      GTK_SELECTION_SINGLE);
    egg_icon_list_set_pixbuf_column (EGG_ICON_LIST (icon_list), 0);
    egg_icon_list_set_text_column (EGG_ICON_LIST (icon_list), 1);

    scrolled_window = get_glade_widget ("icon_list_scrolledwindow");
    gtk_container_add (GTK_CONTAINER (scrolled_window), icon_list);
    
  }  

  /* for all path and associed name */
  egg_icon_list_unselect_all (EGG_ICON_LIST (icon_list));
  menu = gtk_menu_new();
  for (parser_path = icons_path, parser_name = path_name; 
       *parser_path && *parser_name;
       parser_path ++, parser_name++) {
    item = gtk_menu_item_new_with_label(*parser_name);
    g_object_set_data(G_OBJECT(item), "user_data", *parser_path);
    g_object_set_data(G_OBJECT(item), "icon_list", icon_list);
    g_signal_connect_swapped(GTK_OBJECT(item), "activate", 
		     (GCallback) fill_icon_list,
		     item);
    if (parser_path == icons_path)
      fill_icon_list (G_OBJECT(item));
    gtk_widget_show(item);
    gtk_menu_append(GTK_MENU(menu), item);
  }
  item = gtk_menu_item_new_with_label(_("Browse custom directory..."));
  g_signal_connect(GTK_OBJECT(item), "activate", 
		   (GCallback) show_directory_picker,
		   NULL);
  gtk_menu_append(GTK_MENU(menu), item);
  gtk_widget_show(menu);
  option_path = GTK_OPTION_MENU(get_glade_widget( "optionmenu_path"));
  gtk_option_menu_set_menu(option_path, menu);
  gtk_widget_show_all (selector);

}

void
on_toolbar_visible_activate             (GtkWidget     *widget)
{
  GtkWidget * real_widget = get_glade_widget("handlebox");
  if (GTK_WIDGET_MAPPED(GTK_WIDGET(real_widget)))
    gtk_widget_hide(GTK_WIDGET(real_widget));
  else
    gtk_widget_show(GTK_WIDGET(real_widget));
}

void
on_application_visible_activate             (GtkWidget     *widget)
{
   GtkWidget * real_widget = get_glade_widget("scrolledwindow_sys");
  if (GTK_WIDGET_MAPPED(GTK_WIDGET(real_widget)))
    gtk_widget_hide(GTK_WIDGET(real_widget));
  else
    gtk_widget_show(GTK_WIDGET(real_widget));
}

void
on_attribute_visible_activate             (GtkWidget     *widget)
{
  GtkWidget * real_widget = get_glade_widget("notebook_property");
  if (GTK_WIDGET_MAPPED(GTK_WIDGET(real_widget)))
    gtk_widget_hide(GTK_WIDGET(real_widget));
  else
    gtk_widget_show(GTK_WIDGET(real_widget));
}



/*********************************************************************************************/
/************************* callback for drag and drop ****************************************/
/*********************************************************************************************/


GData** drag_object = NULL;

gboolean
check_drag (GtkCTree        *ctree,
	    GtkCTreeNode    *drag_source,
	    GtkCTreeNode    *drag_target,
	    GtkCListDragPos  insert_pos)
{
  return TRUE;
}


/******************************************************************/
/* DND Callbacks                                                  */
/******************************************************************/

/* From GtkClist.c */

gboolean
ctree_menu_drag_drop (GtkWidget      *widget,
		      GdkDragContext *context,
		      gint            x,
		      gint            y,
		      guint           time)
{
  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_CLIST (widget), FALSE);
  g_return_val_if_fail (context != NULL, FALSE);
  return FALSE;
}


/* My callback */


void  
ctree_apps_drag_begin      (GtkWidget *widget,
			      GdkDragContext *drag_context,
			      gpointer user_data)
{
  GtkWidget* clist;
  clist = get_glade_widget("ctree_menu");
  GTK_CLIST_UNSET_FLAG (GTK_CLIST(clist), CLIST_REORDERABLE);
}

void        ctree_apps_drag_end      (GtkWidget *widget,
					GdkDragContext *drag_context,
					gpointer user_data)
{
  GtkWidget* clist;

  clist = get_glade_widget("ctree_menu");
  GTK_CLIST_SET_FLAG (GTK_CLIST(clist), CLIST_REORDERABLE);
}

void         ctree_apps_drag_data_delete      (GtkWidget *widget,
						 GdkDragContext *drag_context,
						 gpointer user_data)
{
  drag_object = NULL;
}

void         ctree_menu_drag_leave      (GtkWidget *widget,
					 GdkDragContext *drag_context,
					 guint time,
					 gpointer user_data)
{
  GtkCList* clist;
  GtkCListDestInfo	*dest_info;
  if (gtk_drag_get_source_widget (drag_context) == widget)
    return ;
  
  clist = GTK_CLIST(widget);	

  dest_info = g_dataset_get_data (drag_context, "gtk-clist-drag-dest");

  /* remove drag line/rectangle */
  if (dest_info->cell.row >= 0) {
    /* for a CTree, one can use GTK_CLIST_ROW(GtkCTreeNode) 
     * as 2nd argument */
    dnd_draw_drag_highlight
      (clist, GTK_CLIST_ROW (ROW_ELEMENT (clist, dest_info->cell.row)),
       dest_info->cell.row, dest_info->insert_pos);
  }
	
  /* init dest_info */
  /* CHECK : is there always a leave signal (after button release for instance ? */
  dest_info->insert_pos  = GTK_CLIST_DRAG_NONE;
  dest_info->cell.row    = -1;

}


/*adapted from gtkctree */
gint
ctree_menu_drag_motion (GtkWidget      *widget,
			GdkDragContext *context,
			gint            x,
			gint            y,
			guint           time,
			gpointer user_data)
{
  GtkCList *clist;
  GtkCTree *ctree;
  GtkCListDestInfo new_info;
  GtkCListDestInfo *dest_info;
  GtkCTreeNode *drag_target;
  
  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_CTREE (widget), FALSE);

  if (gtk_drag_get_source_widget (context) == widget)
    return FALSE;

  clist = GTK_CLIST (widget);
  ctree = GTK_CTREE (widget);

  dest_info = g_dataset_get_data (context, "gtk-clist-drag-dest");

  if (!dest_info)
    {
      dest_info = g_new (GtkCListDestInfo, 1);
      
      dest_info->cell.row    = -1;
      dest_info->cell.column = -1;
      dest_info->insert_pos  = GTK_CLIST_DRAG_NONE;
      
      g_dataset_set_data_full (context, "gtk-clist-drag-dest", dest_info,
			       drag_dest_info_destroy);
    }

  drag_dest_cell (clist, x, y, &new_info);

  drag_target = GTK_CTREE_NODE (g_list_nth (clist->row_list,
					    new_info.cell.row));
      
  if (new_info.cell.row != dest_info->cell.row ||
      (new_info.cell.row == dest_info->cell.row &&
       dest_info->insert_pos != new_info.insert_pos))
    {
      if (dest_info->cell.row >= 0) {
	dnd_draw_drag_highlight (clist,
       g_list_nth (clist->row_list, dest_info->cell.row)->data,
       dest_info->cell.row, dest_info->insert_pos);
      }
      
      dest_info->insert_pos  = new_info.insert_pos;
      dest_info->cell.row    = new_info.cell.row;
      dest_info->cell.column = new_info.cell.column;

      dnd_draw_drag_highlight (clist,
	 g_list_nth (clist->row_list, dest_info->cell.row)->data,
	 dest_info->cell.row, dest_info->insert_pos);
      
      gdk_drag_status (context, context->suggested_action, time);
    }
  return TRUE;
}

void         ctree_apps_drag_data_get      (GtkWidget *widget,
					     GdkDragContext *drag_context,
					     GtkSelectionData *data,
					     guint info,
					     guint time,
					     gpointer user_data)
{
  drag_object = get_selected_object(GTK_CTREE(widget));
}

void         ctree_menu_drag_data_received      (GtkWidget *widget,
						 GdkDragContext *drag_context,
						 gint x,
						 gint y,
						 GtkSelectionData *data,
						 guint info,
						 guint time,
						 gpointer user_data)
{
  GtkCTree *ctree;
  GtkCList *clist;

  GtkCListCellInfo *source_info;
  GData** copy;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_CTREE (widget));
  g_return_if_fail (GTK_IS_CLIST (widget));
  g_return_if_fail (drag_context != NULL);
  g_return_if_fail (data != NULL);
  
  if (drag_object == NULL) 
    return;

  if (gtk_drag_get_source_widget (drag_context) == widget)
    return;
  ctree = GTK_CTREE (widget);
  clist = GTK_CLIST (widget);

  copy = copy_obj (drag_object);

  if (g_ascii_strcasecmp(current_needs, NEEDS(copy))) {
    REMOVE(copy,"needs");
    if (g_ascii_strcasecmp(current_needs, "all"))
      g_datalist_set_data_full(copy,"needs", g_strdup(current_needs), g_free);
    else
      g_datalist_set_data_full(copy,"needs", g_strdup("x11"), g_free);
  }
  
  source_info = (GtkCListCellInfo *)(data->data);
  if (source_info)
    {
      GtkCListDestInfo dest_info;
      GtkCTreeNode *source_node;
      GtkCTreeNode *dest_node;
      
      drag_dest_cell (clist, x, y, &dest_info);
      
      dest_node = GTK_CTREE_NODE (g_list_nth (clist->row_list,
					      dest_info.cell.row));
      gtk_clist_unselect_all (GTK_CLIST(get_glade_widget( "ctree_menu")));
      source_node = insert_obj(GTK_CTREE(get_glade_widget( "ctree_menu")), copy);
      gtk_clist_sort(clist);
      
      if (!source_node || !dest_node)
	return;
      
      switch (dest_info.insert_pos)
	{
	case GTK_CLIST_DRAG_NONE:
	  break;
	case GTK_CLIST_DRAG_INTO:
	  if (check_drag (ctree, source_node, dest_node,
			  dest_info.insert_pos))
	    gtk_ctree_move (ctree, source_node, dest_node,
			    GTK_CTREE_ROW (dest_node)->children);
	  g_dataset_remove_data (drag_context, "gtk-clist-drag-dest");
	  break;
	case GTK_CLIST_DRAG_BEFORE:
	  if (check_drag (ctree, source_node, dest_node,
			  dest_info.insert_pos))
	    gtk_ctree_move (ctree, source_node,
			    GTK_CTREE_ROW (dest_node)->parent, dest_node);
	  g_dataset_remove_data (drag_context, "gtk-clist-drag-dest");
	  break;
	case GTK_CLIST_DRAG_AFTER:
	  if (check_drag (ctree, source_node, dest_node,
			  dest_info.insert_pos)) {
	    if (GTK_CTREE_ROW (dest_node)->sibling != source_node)
	      gtk_ctree_move (ctree, source_node,
			      GTK_CTREE_ROW (dest_node)->parent, 
			      GTK_CTREE_ROW (dest_node)->sibling);
	    else
	      gtk_ctree_move (ctree, source_node,
			      GTK_CTREE_ROW (dest_node)->parent, 
			      NULL);	      
	  }
	  g_dataset_remove_data (drag_context, "gtk-clist-drag-dest");
	  break;
	}
    }
}

static 
void launch_url(char * url) {
  const char * browser;
  GError *error = NULL;
  char * command;
  if (!getuid() && !g_getenv("INITIAL_USER")) {
    show_modal_warning_dialog(_("Security warning : menudrake won't start browser as root"));
    return;
  }
  if ((browser = g_getenv("BROWSER"))) {
    if (! getuid() && g_getenv("INITIAL_USER")) {
	command = g_strdup_printf("/bin/su %s -c /bin/sh %s %s",g_getenv("INITIAL_USER"), browser, url);
    }
    else {
      command = g_strconcat (browser, " ",url,NULL);
    }
    g_spawn_command_line_async (command, &error);    
    g_free (command);
    
    if (error != NULL) {
      g_error_free (error);
    }
  } else {
    /* error */
    show_modal_warning_dialog(_("Unable to run browser"));
  }
}

void
on_mandrakeexpert_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  launch_url("http://www.mandrivaexpert.com/");
}

void
on_help_activate (GtkMenuItem *menuitem,
		  gpointer user_data) 
{
  g_spawn_command_line_async ("drakhelp --id menudrake",
			      NULL);
}


gboolean
on_menudrake_quit                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
 if (is_modified_flag()) {
   GtkWidget * dialog;
   gint result;

   dialog = gtk_message_dialog_new (GTK_WINDOW (get_glade_widget("MenuDrake")),
				    GTK_DIALOG_DESTROY_WITH_PARENT,
				    GTK_MESSAGE_WARNING,
				    GTK_BUTTONS_NONE,
				    _("Menus have not been saved. Do you really want to quit?"));
   gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_YES, GTK_RESPONSE_YES);
   gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_NO, GTK_RESPONSE_NO);
   result = gtk_dialog_run (GTK_DIALOG (dialog));
   if (result == GTK_RESPONSE_YES) {
     gtk_main_quit(); 
   }
   gtk_widget_destroy (dialog);
   return TRUE;
 }
 else {
    gtk_main_quit();
    return FALSE;
  }
}

gboolean
on_menudrake_close_window              (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  on_menudrake_quit (NULL,NULL);
  return TRUE;
}
