/* menudrake.c
 *
 * Copyright (C) 1999, 2000, 2001 vince (Vincent Saugey), MandrakeSoft <vince@mandrakesoft.com>
 *               2001, 2002 Frdric Crozat, MandrakeSoft <fcrozat@mandrakesoft.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <getopt.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <glade/glade.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "callbacks.h"
#include "objmenu.h"

/* Needed for the wm detection */
#include <gdk/gdkx.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

static const struct option longopts[] = {
  {"help",no_argument,NULL,'h'},
  {"usermenu",no_argument,NULL,'u'},
  {"systemmenu",no_argument,NULL,'s'},
  {"menu-style",no_argument,NULL,'m'},
  {NULL, no_argument, NULL, 0}
};

static const char * usage[] = {
  N_("-h	--help		Display this help\n"),
  N_("-u	--usermenu	Edit user menu\n"),
  N_("-s	--systemmenu	Edit System menu\n"),
  N_("-m	--menu-style	Open Menu Style dialog\n"),
  NULL
};

struct wm_info_t {
	char *menu_method;
	char *netwm_id;
};

static struct wm_info_t wm_infos[] =  {
  {"gnome", "Metacity"},
  {"kde", "KWin"},
  {"icewm",""},
  {"sawfish","Sawfish"},
  {"wmaker",""},
  {"afterstep",""},
  {"blackbox",""},
  {"twm",""},
  {"enlightenment",""},
  {"mwm",""},
  {"xfce4","Xfwm4"},
  {NULL,NULL}
};

static GladeXML *xml_tree;
static gchar * current_wm = NULL;

GSList * available_wm;

extern gboolean should_quit;

extern void set_current_needs(gchar * new_needs);

gchar * get_current_wm () {
  return current_wm;
}

GtkWidget * get_glade_widget(char * id) {
  return glade_xml_get_widget(xml_tree,id);
}

static 
char * look_for_wm() {
  const char *name;
  struct wm_info_t *wm;

  name = gdk_x11_screen_get_window_manager_name (gdk_screen_get_default());
  if (name == NULL || strcmp (name, "unknown") == 0)
	  return NULL;

  for (wm = wm_infos; wm->menu_method ; wm++) {
    if (g_ascii_strcasecmp(name, wm->netwm_id) == 0) {
	return wm->menu_method;
    }
  }

  return NULL;
}


static 
void init_interface () {
  GtkWidget *progressbar;
  GSList *lMenus;
  GdkPixmap *banner;
  GdkPixbuf *pixbuf, *side_pixbuf;
  PangoLayout *layout;
  GdkGC *gc;
  gint  width, height, x, y;
  char *text;
  GtkListStore *list_store;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;


  progressbar = get_glade_widget("progressbar");
  gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progressbar), _("Loading menu structure"));
  gtk_window_set_type_hint (GTK_WINDOW(get_glade_widget("progresswindow")), GDK_WINDOW_TYPE_HINT_DIALOG);
    
  gtk_widget_show_all(get_glade_widget("progresswindow"));
    
  gtk_flush();

  if (check_simplified())
     load_translation_map ("/etc/menu-methods/simplified/translate_menus");
  else 
     load_translation_map ("/etc/menu-methods/translate_menus");

  lMenus = create_menu(is_system_menu_flag());
  if (is_system_menu_flag()) {
    gtk_widget_hide(get_glade_widget("follow_system_settings_radio_button"));
  }

  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(progressbar), 0.0);
  set_global_menu_list(lMenus, progressbar);

  gtk_flush();
  gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progressbar), _("Loading available applications"));
  if (is_system_menu_flag()) {
    lMenus = create_lib_menu();
  }
  else {
    lMenus = create_menu(TRUE);
  }
  set_global_app_list(lMenus, progressbar,0.5);
    
  /* let's render text for meny style dialog */
  gtk_window_get_size (GTK_WINDOW (get_glade_widget("menu_style_dialog")),&width,&height);
  pixbuf = gdk_pixbuf_new_from_file (PACKAGE_DATA_DIR "/tranche-menudrake.png", NULL);

  side_pixbuf = gdk_pixbuf_scale_simple (pixbuf, width, 100, GDK_INTERP_BILINEAR);
  g_object_unref (pixbuf);
  
  pixbuf = gdk_pixbuf_new_from_file (PACKAGE_DATA_DIR "/wizard-menudrake.png", NULL);

  gc = gtk_widget_get_style (get_glade_widget("progresswindow"))->text_gc[GTK_STATE_NORMAL];
  banner = gdk_pixmap_new (get_glade_widget("progresswindow")->window, width, 100,-1);
  
  gdk_draw_pixbuf (GDK_DRAWABLE (banner), gc, 
		  side_pixbuf, 0,0,0,0,-1, -1,GDK_RGB_DITHER_NORMAL, 0, 0);
  g_object_unref (side_pixbuf);

  gdk_draw_pixbuf (GDK_DRAWABLE (banner), gc, 
		  pixbuf, 0,0,0,0,-1, -1,GDK_RGB_DITHER_NORMAL, 0, 0);
  layout = pango_layout_new (gtk_widget_get_pango_context (get_glade_widget("progresswindow")));
  text = g_strconcat ("<span font_desc=\"sans 20\" foreground=\"#21449c\">",_("Menu Style"), "</span>", NULL);
  pango_layout_set_markup (layout, text, -1);
  g_free (text);
  pango_layout_get_pixel_size (layout, &width, &height);
  x = 100;
  y = (100 - height) / 2;
  gdk_draw_layout(banner, gc, x, y, layout);
  gtk_image_set_from_pixmap (GTK_IMAGE (get_glade_widget("menu_style_pixmap")),
                             banner, NULL);

  g_object_unref (layout);
  g_object_unref (pixbuf);
  gtk_widget_hide_all(get_glade_widget("progresswindow"));


  list_store = gtk_list_store_new (EXTENDED_N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);

  gtk_tree_view_set_model (GTK_TREE_VIEW (get_glade_widget("list_extended_arg")), 
		  GTK_TREE_MODEL (list_store));
  g_object_unref (G_OBJECT (list_store));

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Attributes"),
	                                             renderer,
                                                     "text", EXTENDED_ATTRIBUTES_COLUMN,
                                                     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (get_glade_widget("list_extended_arg")), column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Values"),
	                                             renderer,
                                                     "text", EXTENDED_VALUES_COLUMN,
                                                     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (get_glade_widget("list_extended_arg")), column);

  g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (get_glade_widget("list_extended_arg")))), 
		  "changed",
                    G_CALLBACK (on_list_extended_arg_select_row),
                    NULL);

}

int
main (int argc, char *argv[]) {
  register int optc;
  char bypass_root_dialog = '\0';
  char * name;
  struct stat s;
  struct wm_info_t *info_p;
  GSList * iterator;
  
#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);  
  textdomain (GETTEXT_PACKAGE);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  bind_textdomain_codeset (MENU_DOMAIN, "UTF-8");
#endif

  gtk_set_locale ();
  gtk_init (&argc, &argv);
  glade_init();

  gtk_window_set_default_icon_from_file (PACKAGE_DATA_DIR "/menudrake48.png", NULL);
  
  while ((optc = getopt_long (argc, argv, "husm", longopts, NULL)) != -1) {
    switch (optc) {
    case 'h':
      {
      char const * const *p; 

      for (p = usage; *p; p++)
	g_print(**p ? _(*p) : *p);
      }
      exit(0);
    case 'u':
      bypass_root_dialog = 'u';
      break;
    case 's':
      bypass_root_dialog = 's';
      break;
    case 'm':
      break;
    }
  }


  if (!(xml_tree = glade_xml_new(PACKAGE_DATA_DIR "/menudrake.glade", NULL, PACKAGE))) {
    g_warning(_("Unable to find menudrake.glade file"));
    exit(1);
  }

  /* FIXME
  gtk_toolbar_set_button_relief(GTK_TOOLBAR(get_glade_widget("toolsbar")),GTK_RELIEF_NONE); */
  /* grr, need to set real title to applications ctree, libglade doesn't show it */
  gtk_clist_set_column_title (GTK_CLIST(get_glade_widget("ctree_apps")), 0, _("Available applications"));



  /* connect the signals in the interface */
  glade_xml_signal_autoconnect(xml_tree);

  /* let's try to find which wm is installed */
  available_wm = NULL;
  for (info_p = wm_infos ; info_p->menu_method ; info_p++) {
    name = g_strdup_printf("/usr/X11R6/bin/start%s", info_p->menu_method);
    if (stat (name, &s)) {
      g_free (name);
      name = g_strdup_printf("/usr/bin/start%s",info_p->menu_method);
      if (stat (name, &s)) {
	available_wm = g_slist_append(available_wm, g_strdup(""));
      }
      else {
	available_wm =  g_slist_append(available_wm, g_strdup(info_p->menu_method));
      }
    }
    else {
      available_wm = g_slist_append(available_wm, g_strdup(info_p->menu_method));
    }
    g_free (name);
  }

  if (current_wm == NULL) {
    current_wm = look_for_wm ();

  }
  if (current_wm != NULL) {
    set_current_needs(current_wm);
  }
  else {
    set_current_needs("all");
  }
   
  if (!getuid() && !bypass_root_dialog) {
      gtk_widget_show_all(get_glade_widget("rootmenu_dialog"));
      gtk_main();
      if (should_quit)
	exit(0);
  }
  else { /* must be called before displaying main window */
    set_system_menu_flag(FALSE);
  }

  gtk_widget_show_all(get_glade_widget("MenuDrake"));

  if (getuid()==0) {
    switch (bypass_root_dialog) {
    case 'u':
      on_root_menu_clicked(NULL,NULL);
      break;
    case 's':
      on_system_menu_clicked(NULL,NULL);
      break;
    default:
      break;
    }
  }
  else { /* hide toggle buttons between system and root menu */
    gtk_widget_hide_all(get_glade_widget("rootmenu_vbox"));  
  }


  gtk_flush();
 
  init_interface();
  gtk_main ();

  for (iterator = available_wm ; iterator ; iterator = g_slist_next(iterator)) {
    g_free(iterator->data);
  }
  g_slist_free(available_wm);

  exit (0);
}

#if 0
/* placeholder, not compiled */

char *foo[] {
/* translate this the same way as in menu-message package */
N_("-> All Applications"),
/* translate this the same way as in menu-message package */
N_("-> What to do?"),
}
#endif
